@@ -95,12 +95,14 @@ completion_free (t_completion *completion)
         free (completion->args);
     completion->args = NULL;
     
-    while (completion->completion_list)
-        weelist_remove (&completion->completion_list,
-                        &completion->last_completion,
-                        completion->completion_list);
+    weelist_remove_all (&completion->completion_list,
+                        &completion->last_completion);
     completion->completion_list = NULL;
     completion->last_completion = NULL;
+
+    if (completion->word_found)
+        free (completion->word_found);
+    completion->word_found = NULL;
 }
 
 /*
@@ -203,20 +205,96 @@ completion_get_command_infos (t_completion *completion,
     return;
 }
 
+/*
+ * completion_is_only_alphanum: return 1 if there is only alpha/num chars
+ *                              in a string
+ */
+
+int
+completion_is_only_alphanum (char *string)
+{
+    while (string[0])
+    {
+        if (strchr (cfg_look_nick_completion_ignore, string[0]))
+            return 0;
+        string++;
+    }
+    return 1;
+}
+
+/*
+ * completion_strdup_alphanum: duplicate alpha/num chars in a string
+ */
+
+char *
+completion_strdup_alphanum (char *string)
+{
+    char *result, *pos;
+    
+    result = (char *)malloc (strlen (string) + 1);
+    pos = result;
+    while (string[0])
+    {
+        if (!strchr (cfg_look_nick_completion_ignore, string[0]))
+        {
+            pos[0] = string[0];
+            pos++;
+        }
+        string++;
+    }
+    pos[0] = '\0';
+    return result;
+}
+
+/*
+ * completion_nickncmp: locale and case independent string comparison
+ *                      with max length for nicks (alpha or digits only)
+ */
+
+int
+completion_nickncmp (char *base_word, char *nick, int max)
+{
+    char *base_word2, *nick2;
+    int return_cmp;
+    
+    if (!cfg_look_nick_completion_ignore
+        || !cfg_look_nick_completion_ignore[0]
+        || !base_word || !nick || !base_word[0] || !nick[0]
+        || (!completion_is_only_alphanum (base_word)))
+        return ascii_strncasecmp (base_word, nick, max);
+    
+    base_word2 = completion_strdup_alphanum (base_word);
+    nick2 = completion_strdup_alphanum (nick);
+    
+    return_cmp = ascii_strncasecmp (base_word2, nick2, strlen (base_word2));
+    
+    free (base_word2);
+    free (nick2);
+    
+    return return_cmp;
+}
+
 /*
  * completion_list_add: add a word to completion word list
  */
 
 void
-completion_list_add (t_completion *completion, char *word)
+completion_list_add (t_completion *completion, char *word,
+                     int nick_completion, int position)
 {
+    if (!word || !word[0])
+        return;
+    
     if (!completion->base_word || !completion->base_word[0]
-        || (ascii_strncasecmp (completion->base_word, word,
-                               strlen (completion->base_word)) == 0))
+        || (nick_completion && (completion_nickncmp (completion->base_word, word,
+                                                            strlen (completion->base_word)) == 0))
+        || (!nick_completion && (ascii_strncasecmp (completion->base_word, word,
+                                                          strlen (completion->base_word)) == 0)))
     {
         weelist_add (&completion->completion_list,
                      &completion->last_completion,
-                     word);
+                     word,
+                     position);
     }
 }
 
@@ -231,7 +309,8 @@ completion_list_add_alias (t_completion *completion)
     
     for (ptr_alias = weechat_alias; ptr_alias; ptr_alias = ptr_alias->next_alias)
     {
-        completion_list_add (completion, ptr_alias->alias_name);
+        completion_list_add (completion, ptr_alias->alias_name,
+                             0, WEELIST_POS_SORT);
     }
 }
 
@@ -246,7 +325,7 @@ completion_list_add_alias_cmd (t_completion *completion)
     
     for (ptr_list = index_commands; ptr_list; ptr_list = ptr_list->next_weelist)
     {
-        completion_list_add (completion, ptr_list->data);
+        completion_list_add (completion, ptr_list->data, 0, WEELIST_POS_SORT);
     }
 }
 
@@ -259,24 +338,31 @@ completion_list_add_channel (t_completion *completion)
 {
     if (completion->channel)
         completion_list_add (completion,
-                             ((t_irc_channel *)(completion->channel))->name);
+                             ((t_irc_channel *)(completion->channel))->name,
+                             0, WEELIST_POS_SORT);
 }
 
 /*
- * completion_list_add_server_channels: add server channels to completion list
+ * completion_list_add_channels: add server channels to completion list
  */
 
 void
-completion_list_add_server_channels (t_completion *completion)
+completion_list_add_channels (t_completion *completion)
 {
+    t_irc_server *ptr_server;
     t_irc_channel *ptr_channel;
     
     if (completion->server)
     {
-        for (ptr_channel = ((t_irc_server *)(completion->server))->channels;
-             ptr_channel; ptr_channel = ptr_channel->next_channel)
+        for (ptr_server = irc_servers; ptr_server;
+             ptr_server = ptr_server->next_server)
         {
-            completion_list_add (completion, ptr_channel->name);
+            for (ptr_channel = ptr_server->channels;
+                 ptr_channel; ptr_channel = ptr_channel->next_channel)
+            {
+                completion_list_add (completion, ptr_channel->name,
+                                     0, WEELIST_POS_SORT);
+            }
         }
     }
 }
@@ -361,7 +447,8 @@ completion_list_add_filename (t_completion *completion)
 			 entry->d_name, 
 			 S_ISDIR(statbuf.st_mode) ? DIR_SEPARATOR : "");
 		
-		completion_list_add (completion, buffer);
+		completion_list_add (completion, buffer,
+                                     0, WEELIST_POS_SORT);
 	    }
 	}
     }
@@ -392,11 +479,12 @@ completion_list_add_plugin_cmd (t_completion *completion)
              ptr_handler; ptr_handler = ptr_handler->next_handler)
         {
             if (ptr_handler->type == PLUGIN_HANDLER_COMMAND)
-                completion_list_add (completion, ptr_handler->command);
+                completion_list_add (completion, ptr_handler->command,
+                                     0, WEELIST_POS_SORT);
         }
     }
 #else
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) completion;
 #endif
 }
@@ -413,7 +501,8 @@ completion_list_add_irc_cmd_sent (t_completion *completion)
     for (i = 0; irc_commands[i].command_name; i++)
     {
         if (irc_commands[i].cmd_function_args || irc_commands[i].cmd_function_1arg)
-            completion_list_add (completion, irc_commands[i].command_name);
+            completion_list_add (completion, irc_commands[i].command_name,
+                                 0, WEELIST_POS_SORT);
     }
 }
 
@@ -429,7 +518,8 @@ completion_list_add_irc_cmd_recv (t_completion *completion)
     for (i = 0; irc_commands[i].command_name; i++)
     {
         if (irc_commands[i].recv_function)
-            completion_list_add(completion, irc_commands[i].command_name);
+            completion_list_add(completion, irc_commands[i].command_name,
+                                0, WEELIST_POS_SORT);
     }
 }
 
@@ -444,7 +534,8 @@ completion_list_add_key_cmd (t_completion *completion)
     
     for (i = 0; gui_key_functions[i].function_name; i++)
     {
-        completion_list_add (completion, gui_key_functions[i].function_name);
+        completion_list_add (completion, gui_key_functions[i].function_name,
+                             0, WEELIST_POS_SORT);
     }
 }
 
@@ -455,9 +546,62 @@ completion_list_add_key_cmd (t_completion *completion)
 void
 completion_list_add_self_nick (t_completion *completion)
 {
+    if (completion->server)
+        completion_list_add (completion,
+                             ((t_irc_server *)(completion->server))->nick,
+                             0, WEELIST_POS_SORT);
+}
+
+/*
+ * completion_list_add_server_nicks: add server nicks to completion list
+ */
+
+void
+completion_list_add_server_nicks (t_completion *completion)
+{
+    t_irc_server *ptr_server;
+    t_irc_channel *ptr_channel;
+    t_irc_nick *ptr_nick;
+    
     if (completion->server)
     {
-        completion_list_add (completion, ((t_irc_server *)(completion->server))->nick);
+        for (ptr_server = (t_irc_server *)(completion->server); ptr_server;
+             ptr_server = ptr_server->next_server)
+        {
+            for (ptr_channel = ptr_server->channels; ptr_channel;
+                 ptr_channel = ptr_channel->next_channel)
+            {
+                if ((!completion->channel || (t_irc_channel *)(completion->channel) != ptr_channel)
+                    && (ptr_channel->type == CHANNEL_TYPE_CHANNEL))
+                {
+                    for (ptr_nick = ptr_channel->nicks; ptr_nick;
+                         ptr_nick = ptr_nick->next_nick)
+                    {
+                        completion_list_add (completion, ptr_nick->nick,
+                                             1, WEELIST_POS_SORT);
+                    }
+                }
+            }
+        }
+        
+        /* add current channel nicks at beginning */
+        if (completion->channel && (((t_irc_channel *)(completion->channel))->type == CHANNEL_TYPE_CHANNEL))
+        {
+            for (ptr_nick = ((t_irc_channel *)(completion->channel))->nicks;
+                 ptr_nick; ptr_nick = ptr_nick->next_nick)
+            {
+                completion_list_add (completion, ptr_nick->nick,
+                                     1, WEELIST_POS_BEGINNING);
+            }
+        }
+        
+        /* add self nick at the end */
+        if (completion->server)
+            completion_list_add (completion,
+                                 ((t_irc_server *)(completion->server))->nick,
+                                 1, WEELIST_POS_END);
+        
+        completion->arg_is_nick = 1;
     }
 }
 
@@ -469,22 +613,45 @@ void
 completion_list_add_channel_nicks (t_completion *completion)
 {
     t_irc_nick *ptr_nick;
+    t_weelist *ptr_weelist;
     
     if (completion->channel)
     {
         if (((t_irc_channel *)(completion->channel))->type == CHANNEL_TYPE_CHANNEL)
         {
+            /* add channel nicks */
             for (ptr_nick = ((t_irc_channel *)(completion->channel))->nicks;
                  ptr_nick; ptr_nick = ptr_nick->next_nick)
             {
-                completion_list_add (completion, ptr_nick->nick);
+                completion_list_add (completion, ptr_nick->nick,
+                                     1, WEELIST_POS_SORT);
             }
+            
+            /* add nicks speaking recently on this channel */
+            if (cfg_look_nick_completion_smart)
+            {
+                for (ptr_weelist = ((t_irc_channel *)(completion->channel))->nicks_speaking;
+                     ptr_weelist; ptr_weelist = ptr_weelist->next_weelist)
+                {
+                    if (irc_nick_search ((t_irc_channel *)(completion->channel),
+                                         ptr_weelist->data))
+                        completion_list_add (completion, ptr_weelist->data,
+                                             1, WEELIST_POS_BEGINNING);
+                }
+            }
+            
+            /* add self nick at the end */
+            if (completion->server)
+                completion_list_add (completion,
+                                     ((t_irc_server *)(completion->server))->nick,
+                                     1, WEELIST_POS_END);
         }
         if ((((t_irc_channel *)(completion->channel))->type == CHANNEL_TYPE_PRIVATE)
             || (((t_irc_channel *)(completion->channel))->type == CHANNEL_TYPE_DCC_CHAT))
         {
             completion_list_add (completion,
-                                 ((t_irc_channel *)(completion->channel))->name);
+                                 ((t_irc_channel *)(completion->channel))->name,
+                                 1, WEELIST_POS_SORT);
         }
         completion->arg_is_nick = 1;
     }
@@ -508,7 +675,8 @@ completion_list_add_channel_nicks_hosts (t_completion *completion)
             for (ptr_nick = ((t_irc_channel *)(completion->channel))->nicks;
                  ptr_nick; ptr_nick = ptr_nick->next_nick)
             {
-                completion_list_add (completion, ptr_nick->nick);
+                completion_list_add (completion, ptr_nick->nick,
+                                     1, WEELIST_POS_SORT);
                 if (ptr_nick->host)
                 {
                     length = strlen (ptr_nick->nick) + 1 +
@@ -518,7 +686,8 @@ completion_list_add_channel_nicks_hosts (t_completion *completion)
                     {
                         snprintf (buf, length, "%s!%s",
                                   ptr_nick->nick, ptr_nick->host);
-                        completion_list_add (completion, buf);
+                        completion_list_add (completion, buf,
+                                             1, WEELIST_POS_SORT);
                         free (buf);
                     }
                 }
@@ -528,7 +697,8 @@ completion_list_add_channel_nicks_hosts (t_completion *completion)
              || (((t_irc_channel *)(completion->channel))->type == CHANNEL_TYPE_PRIVATE))
         {
             completion_list_add (completion,
-                                 ((t_irc_channel *)(completion->channel))->name);
+                                 ((t_irc_channel *)(completion->channel))->name,
+                                 1, WEELIST_POS_SORT);
         }
         completion->arg_is_nick = 1;
     }
@@ -553,7 +723,8 @@ completion_list_add_option (t_completion *completion)
             for (j = 0; weechat_options[i][j].option_name; j++)
             {
                 completion_list_add (completion,
-                                     weechat_options[i][j].option_name);
+                                     weechat_options[i][j].option_name,
+                                     0, WEELIST_POS_SORT);
             }
         }
     }
@@ -565,7 +736,7 @@ completion_list_add_option (t_completion *completion)
             snprintf (option_name, sizeof (option_name), "%s.%s",
                       ptr_server->name, 
                       weechat_options[CONFIG_SECTION_SERVER][i].option_name);
-            completion_list_add (completion, option_name);
+            completion_list_add (completion, option_name, 0, WEELIST_POS_SORT);
         }
     }
 }
@@ -583,10 +754,10 @@ completion_list_add_plugin_option (t_completion *completion)
     for (ptr_option = plugin_options; ptr_option;
          ptr_option = ptr_option->next_option)
     {
-        completion_list_add (completion, ptr_option->name);
+        completion_list_add (completion, ptr_option->name, 0, WEELIST_POS_SORT);
     }
 #else
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) completion;
 #endif
 }
@@ -599,7 +770,8 @@ void
 completion_list_add_part (t_completion *completion)
 {
     if (cfg_irc_default_msg_part && cfg_irc_default_msg_part[0])
-        completion_list_add (completion, cfg_irc_default_msg_part);
+        completion_list_add (completion, cfg_irc_default_msg_part,
+                             0, WEELIST_POS_SORT);
 }
 
 /*
@@ -615,10 +787,11 @@ completion_list_add_plugin (t_completion *completion)
     for (ptr_plugin = weechat_plugins; ptr_plugin;
          ptr_plugin = ptr_plugin->next_plugin)
     {
-        completion_list_add (completion, ptr_plugin->name);
+        completion_list_add (completion, ptr_plugin->name,
+                             0, WEELIST_POS_SORT);
     }
 #else
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) completion;
 #endif
 }
@@ -631,7 +804,8 @@ void
 completion_list_add_quit (t_completion *completion)
 {
     if (cfg_irc_default_msg_quit && cfg_irc_default_msg_quit[0])
-        completion_list_add (completion, cfg_irc_default_msg_quit);
+        completion_list_add (completion, cfg_irc_default_msg_quit,
+                             0, WEELIST_POS_SORT);
 }
 
 /*
@@ -643,7 +817,8 @@ completion_list_add_server (t_completion *completion)
 {
     if (completion->server)
         completion_list_add (completion,
-                             ((t_irc_server *)(completion->server))->name);
+                             ((t_irc_server *)(completion->server))->name,
+                             0, WEELIST_POS_SORT);
 }
 
 /*
@@ -658,7 +833,8 @@ completion_list_add_servers (t_completion *completion)
     for (ptr_server = irc_servers; ptr_server;
          ptr_server = ptr_server->next_server)
     {
-        completion_list_add (completion, ptr_server->name);
+        completion_list_add (completion, ptr_server->name,
+                             0, WEELIST_POS_SORT);
     }
 }
 
@@ -671,11 +847,9 @@ completion_list_add_topic (t_completion *completion)
 {
     char *string;
     
-    if (!completion->server || !completion->channel
-        || !((t_irc_channel *)(completion->channel))->topic
-        || !((t_irc_channel *)(completion->channel))->topic[0])
-        completion_stop (completion);
-    else
+    if (completion->server && completion->channel
+        && ((t_irc_channel *)(completion->channel))->topic
+        && ((t_irc_channel *)(completion->channel))->topic[0])
     {
         if (cfg_irc_colors_send)
             string = (char *)gui_color_decode_for_user_entry ((unsigned char *)((t_irc_channel *)(completion->channel))->topic);
@@ -683,7 +857,8 @@ completion_list_add_topic (t_completion *completion)
             string = (char *)gui_color_decode ((unsigned char *)((t_irc_channel *)(completion->channel))->topic, 0);
         completion_list_add (completion,
                              (string) ?
-                             string : ((t_irc_channel *)(completion->channel))->topic);
+                             string : ((t_irc_channel *)(completion->channel))->topic,
+                             0, WEELIST_POS_SORT);
         if (string)
             free (string);
     }
@@ -715,26 +890,31 @@ completion_list_add_option_value (t_completion *completion)
             {
                 case OPTION_TYPE_BOOLEAN:
                     if (option_value && (*((int *)(option_value))))
-                        completion_list_add (completion, "on");
+                        completion_list_add (completion, "on",
+                                             0, WEELIST_POS_SORT);
                     else
-                        completion_list_add (completion, "off");
+                        completion_list_add (completion, "off",
+                                             0, WEELIST_POS_SORT);
                     break;
                 case OPTION_TYPE_INT:
                     snprintf (option_string, sizeof (option_string) - 1,
                               "%d", (option_value) ? *((int *)(option_value)) : option->default_int);
-                    completion_list_add (completion, option_string);
+                    completion_list_add (completion, option_string,
+                                         0, WEELIST_POS_SORT);
                     break;
                 case OPTION_TYPE_INT_WITH_STRING:
                     completion_list_add (completion,
                                          (option_value) ?
                                          option->array_values[*((int *)(option_value))] :
-                                         option->array_values[option->default_int]);
+                                         option->array_values[option->default_int],
+                                         0, WEELIST_POS_SORT);
                     break;
                 case OPTION_TYPE_COLOR:
                     completion_list_add (completion,
                                          (option_value) ?
                                          gui_color_get_name (*((int *)(option_value))) :
-                                         option->default_string);
+                                         option->default_string,
+                                         0, WEELIST_POS_SORT);
                     break;
                 case OPTION_TYPE_STRING:
                     snprintf (option_string, sizeof (option_string) - 1,
@@ -742,7 +922,8 @@ completion_list_add_option_value (t_completion *completion)
                               ((option_value) && (*((char **)(option_value)))) ?
                               *((char **)(option_value)) :
                               option->default_string);
-                    completion_list_add (completion, option_string);
+                    completion_list_add (completion, option_string,
+                                         0, WEELIST_POS_SORT);
                     break;
             }
         }
@@ -770,13 +951,14 @@ completion_list_add_plugin_option_value (t_completion *completion)
         
         ptr_option = plugin_config_search_internal (completion->args);
         if (ptr_option)
-            completion_list_add (completion, ptr_option->value);
+            completion_list_add (completion, ptr_option->value,
+                                 0, WEELIST_POS_SORT);
         
         if (pos)
             pos[0] = ' ';
     }
 #else
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) completion;
 #endif
 }
@@ -792,7 +974,8 @@ completion_list_add_weechat_cmd (t_completion *completion)
     
     for (i = 0; weechat_commands[i].command_name; i++)
     {
-        completion_list_add (completion, weechat_commands[i].command_name);
+        completion_list_add (completion, weechat_commands[i].command_name,
+                             0, WEELIST_POS_SORT);
     }
 }
 
@@ -821,7 +1004,8 @@ completion_build_list_template (t_completion *completion, char *template)
                     word[word_offset] = '\0';
                     weelist_add (&completion->completion_list,
                                  &completion->last_completion,
-                                 word);
+                                 word,
+                                 WEELIST_POS_SORT);
                 }
                 word_offset = 0;
                 break;
@@ -846,7 +1030,7 @@ completion_build_list_template (t_completion *completion, char *template)
                             completion_list_add_channel (completion);
                             break;
                         case 'C': /* all channels */
-                            completion_list_add_server_channels (completion);
+                            completion_list_add_channels (completion);
                             break;
                         case 'f': /* filename */
                             completion_list_add_filename (completion);
@@ -866,6 +1050,9 @@ completion_build_list_template (t_completion *completion, char *template)
                         case 'm': /* self nickname */
                             completion_list_add_self_nick (completion);
                             break;
+                        case 'M': /* nicks of current server (all open channels) */
+                            completion_list_add_server_nicks (completion);
+                            break;
                         case 'n': /* channel nicks */
                             completion_list_add_channel_nicks (completion);
                             completion->context = COMPLETION_NICK;
@@ -1107,7 +1294,9 @@ completion_command (t_completion *completion)
         {
             if ((!completion->word_found) || word_found_seen)
             {
-                completion->word_found = ptr_weelist->data;
+                if (completion->word_found)
+                    free (completion->word_found);
+                completion->word_found = strdup (ptr_weelist->data);
 
                 if (completion->direction < 0)
                     ptr_weelist2 = ptr_weelist->prev_weelist;
@@ -1146,80 +1335,12 @@ completion_command (t_completion *completion)
     }
     if (completion->word_found)
     {
+        free (completion->word_found);
         completion->word_found = NULL;
         completion_command (completion);
     }
 }
 
-/*
- * completion_is_only_alphanum: return 1 if there is only alpha/num chars
- *                              in a string
- */
-
-int
-completion_is_only_alphanum (char *string)
-{
-    while (string[0])
-    {
-        if (strchr (cfg_look_nick_completion_ignore, string[0]))
-            return 0;
-        string++;
-    }
-    return 1;
-}
-
-/*
- * completion_strdup_alphanum: duplicate alpha/num chars in a string
- */
-
-char *
-completion_strdup_alphanum (char *string)
-{
-    char *result, *pos;
-    
-    result = (char *)malloc (strlen (string) + 1);
-    pos = result;
-    while (string[0])
-    {
-        if (!strchr (cfg_look_nick_completion_ignore, string[0]))
-        {
-            pos[0] = string[0];
-            pos++;
-        }
-        string++;
-    }
-    pos[0] = '\0';
-    return result;
-}
-
-/*
- * completion_nickncmp: locale and case independent string comparison
- *                      with max length for nicks (alpha or digits only)
- */
-
-int
-completion_nickncmp (char *base_word, char *nick, int max)
-{
-    char *base_word2, *nick2;
-    int return_cmp;
-    
-    if (!cfg_look_nick_completion_ignore
-        || !cfg_look_nick_completion_ignore[0]
-        || !base_word || !nick || !base_word[0] || !nick[0]
-        || (!completion_is_only_alphanum (base_word)))
-        return ascii_strncasecmp (base_word, nick, max);
-    
-    base_word2 = completion_strdup_alphanum (base_word);
-    nick2 = completion_strdup_alphanum (nick);
-    
-    return_cmp = ascii_strncasecmp (base_word2, nick2, strlen (base_word2));
-    
-    free (base_word2);
-    free (nick2);
-    
-    return return_cmp;
-}
-
 /*
  * completion_command_arg: complete a command argument
  */
@@ -1245,7 +1366,9 @@ completion_command_arg (t_completion *completion, int nick_completion)
         {
             if ((!completion->word_found) || word_found_seen)
             {
-                completion->word_found = ptr_weelist->data;
+                if (completion->word_found)
+                    free (completion->word_found);
+                completion->word_found = strdup (ptr_weelist->data);
                 
                 if (completion->direction < 0)
                     ptr_weelist2 = ptr_weelist->prev_weelist;
@@ -1286,6 +1409,7 @@ completion_command_arg (t_completion *completion, int nick_completion)
     }
     if (completion->word_found)
     {
+        free (completion->word_found);
         completion->word_found = NULL;
         completion_command_arg (completion, nick_completion);
     }
@@ -1299,7 +1423,8 @@ void
 completion_nick (t_completion *completion)
 {
     int length, word_found_seen, other_completion;
-    t_irc_nick *ptr_nick, *ptr_nick2;
+    t_irc_nick *ptr_nick;
+    t_weelist *ptr_weelist, *ptr_weelist2;
 
     if (!completion->channel)
         return;
@@ -1313,31 +1438,83 @@ completion_nick (t_completion *completion)
         {
             weelist_add (&completion->completion_list,
                          &completion->last_completion,
-                         ((t_irc_channel *)(completion->channel))->name);
+                         ((t_irc_channel *)(completion->channel))->name,
+                         WEELIST_POS_SORT);
             weelist_add (&completion->completion_list,
                          &completion->last_completion,
-                         ((t_irc_server *)(completion->server))->nick);
+                         ((t_irc_server *)(completion->server))->nick,
+                         WEELIST_POS_SORT);
         }
         completion_command_arg (completion, 1);
         return;
     }
+
+    /* rebuild nick list for completion, with nicks speaking at beginning of list */
+    if ((((t_irc_channel *)(completion->channel))->nick_completion_reset)
+        || (!(completion->completion_list)))
+    {
+        /* empty completion list */
+        if (completion->completion_list)
+        {
+            weelist_remove_all (&(completion->completion_list),
+                                &(completion->last_completion));
+            completion->completion_list = NULL;
+            completion->last_completion = NULL;
+        }
+        
+        /* add channel nicks */
+        for (ptr_nick = ((t_irc_channel *)(completion->channel))->nicks;
+             ptr_nick; ptr_nick = ptr_nick->next_nick)
+        {
+            weelist_add (&(completion->completion_list),
+                         &(completion->last_completion),
+                         ptr_nick->nick,
+                         WEELIST_POS_SORT);
+        }
+        
+        /* add nicks speaking recently on this channel */
+        if (cfg_look_nick_completion_smart)
+        {
+            for (ptr_weelist = ((t_irc_channel *)(completion->channel))->nicks_speaking;
+                 ptr_weelist; ptr_weelist = ptr_weelist->next_weelist)
+            {
+                if (irc_nick_search ((t_irc_channel *)(completion->channel),
+                                     ptr_weelist->data))
+                    weelist_add (&(completion->completion_list),
+                                 &(completion->last_completion),
+                                 ptr_weelist->data,
+                                 WEELIST_POS_BEGINNING);
+            }
+        }
+        
+        /* add self nick at the end */
+        if (completion->server)
+            weelist_add (&(completion->completion_list),
+                         &(completion->last_completion),
+                         ((t_irc_server *)(completion->server))->nick,
+                         WEELIST_POS_END);
+        
+        ((t_irc_channel *)(completion->channel))->nick_completion_reset = 0;
+    }
     
     length = strlen (completion->base_word);
     word_found_seen = 0;
     other_completion = 0;
-
-    if (completion->direction < 0)
-        ptr_nick = ((t_irc_channel *)(completion->channel))->last_nick;
-    else
-        ptr_nick = ((t_irc_channel *)(completion->channel))->nicks;
     
-    while (ptr_nick)
+    if (completion->direction < 0)
+        ptr_weelist = completion->last_completion;
+    else
+        ptr_weelist = completion->completion_list;
+    
+    while (ptr_weelist)
     {
-        if (completion_nickncmp (completion->base_word, ptr_nick->nick, length) == 0)
+        if (completion_nickncmp (completion->base_word, ptr_weelist->data, length) == 0)
         {
             if ((!completion->word_found) || word_found_seen)
             {
-                completion->word_found = ptr_nick->nick;
+                if (completion->word_found)
+                    free (completion->word_found);
+                completion->word_found = strdup (ptr_weelist->data);
                 if (cfg_look_nick_complete_first)
                 {
                     completion->position = -1;
@@ -1345,21 +1522,21 @@ completion_nick (t_completion *completion)
                 }
 
                 if (completion->direction < 0)
-                    ptr_nick2 = ptr_nick->prev_nick;
+                    ptr_weelist2 = ptr_weelist->prev_weelist;
                 else
-                    ptr_nick2 = ptr_nick->next_nick;
+                    ptr_weelist2 = ptr_weelist->next_weelist;
                 
-                while (ptr_nick2)
+                while (ptr_weelist2)
                 {
                     if (completion_nickncmp (completion->base_word,
-                                             ptr_nick2->nick,
+                                             ptr_weelist2->data,
                                              length) == 0)
                         other_completion++;
-
+                    
                     if (completion->direction < 0)
-                        ptr_nick2 = ptr_nick2->prev_nick;
+                        ptr_weelist2 = ptr_weelist2->prev_weelist;
                     else
-                        ptr_nick2 = ptr_nick2->next_nick;
+                        ptr_weelist2 = ptr_weelist2->next_weelist;
                 }
                 
                 if (other_completion == 0)
@@ -1374,16 +1551,17 @@ completion_nick (t_completion *completion)
             other_completion++;
         }
         if (completion->word_found &&
-            (ascii_strcasecmp (ptr_nick->nick, completion->word_found) == 0))
+            (ascii_strcasecmp (ptr_weelist->data, completion->word_found) == 0))
             word_found_seen = 1;
-
+        
         if (completion->direction < 0)
-            ptr_nick = ptr_nick->prev_nick;
+            ptr_weelist = ptr_weelist->prev_weelist;
         else
-            ptr_nick = ptr_nick->next_nick;
+            ptr_weelist = ptr_weelist->next_weelist;
     }
     if (completion->word_found)
     {
+        free (completion->word_found);
         completion->word_found = NULL;
         completion_nick (completion);
     }
@@ -1407,10 +1585,10 @@ completion_auto (t_completion *completion)
     }
     
     /* channel completion */
-    if (string_is_channel (completion->base_word))
+    if (irc_channel_is_channel (completion->base_word))
     {
         if (!completion->completion_list)
-            completion_list_add_server_channels (completion);
+            completion_list_add_channels (completion);
         completion_command_arg (completion, 0);
         return;
     }
@@ -1437,12 +1615,15 @@ completion_search (t_completion *completion, int direction,
     /* if new completion => look for base word */
     if (pos != completion->position)
     {
+        if (completion->word_found)
+            free (completion->word_found);
         completion->word_found = NULL;
         completion_find_context (completion, buffer, size, pos);
     }
     
     /* completion */
-    old_word_found = completion->word_found;
+    old_word_found = (completion->word_found) ?
+        strdup (completion->word_found) : NULL;
     switch (completion->context)
     {
         case COMPLETION_NULL:
@@ -1489,6 +1670,8 @@ completion_search (t_completion *completion, int direction,
             }
         }
     }
+    if (old_word_found)
+        free (old_word_found);
 }
 
 /*
