@@ -259,6 +259,7 @@ session_save_servers (FILE *file)
         rc = rc && (session_write_str (file, SESSION_SERV_UNTERMINATED_MESSAGE, ptr_server->unterminated_message));
         rc = rc && (session_write_str (file, SESSION_SERV_NICK, ptr_server->nick));
         rc = rc && (session_write_str (file, SESSION_SERV_NICK_MODES, ptr_server->nick_modes));
+        rc = rc && (session_write_str (file, SESSION_SERV_PREFIX, ptr_server->prefix));
         rc = rc && (session_write_buf (file, SESSION_SERV_RECONNECT_START, &(ptr_server->reconnect_start), sizeof (time_t)));
         rc = rc && (session_write_int (file, SESSION_SERV_RECONNECT_JOIN, ptr_server->reconnect_join));
         rc = rc && (session_write_int (file, SESSION_SERV_IS_AWAY, ptr_server->is_away));
@@ -373,7 +374,7 @@ session_save_line (FILE *file, t_gui_line *line)
     rc = rc && (session_write_int (file, SESSION_LINE_WITH_MESSAGE, line->line_with_message));
     rc = rc && (session_write_int (file, SESSION_LINE_WITH_HIGHLIGHT, line->line_with_highlight));
     rc = rc && (session_write_str (file, SESSION_LINE_DATA, line->data));
-    rc = rc && (session_write_int (file, SESSION_LINE_OFS_AFTER_DATE, line->ofs_start_message));
+    rc = rc && (session_write_int (file, SESSION_LINE_OFS_AFTER_DATE, line->ofs_after_date));
     rc = rc && (session_write_int (file, SESSION_LINE_OFS_START_MESSAGE, line->ofs_start_message));
     rc = rc && (session_write_str (file, SESSION_LINE_NICK, line->nick));
     rc = rc && (session_write_buf (file, SESSION_LINE_DATE, &(line->date), sizeof (time_t)));
@@ -825,7 +826,6 @@ session_load_server (FILE *file)
     void *session_data;
     size_t session_size;
     int session_size_int;
-    const int cert_type_prio[] = { GNUTLS_CRT_X509, GNUTLS_CRT_OPENPGP, 0 };
 #endif
     
     /* read server name */
@@ -839,20 +839,20 @@ session_load_server (FILE *file)
     /* use or allocate server */
     weechat_log_printf (_("session: loading server \"%s\"\n"),
                         server_name);
-    session_current_server = server_search (server_name);
+    session_current_server = irc_server_search (server_name);
     if (session_current_server)
         weechat_log_printf (_("server found, updating values\n"));
     else
     {
         weechat_log_printf (_("server not found, creating new one\n"));
-        session_current_server = server_alloc ();
+        session_current_server = irc_server_alloc ();
         if (!session_current_server)
         {
             free (server_name);
             session_crash (file, _("can't create new server"));
             return 0;
         }
-        server_init (session_current_server);
+        irc_server_init (session_current_server);
         session_current_server->name = strdup (server_name);
     }
     free (server_name);
@@ -958,7 +958,8 @@ session_load_server (FILE *file)
                     return 0;
                 }
                 gnutls_set_default_priority (session_current_server->gnutls_sess);
-                gnutls_certificate_type_set_priority (session_current_server->gnutls_sess, cert_type_prio);
+                gnutls_certificate_type_set_priority (session_current_server->gnutls_sess, gnutls_cert_type_prio);
+                gnutls_protocol_set_priority (session_current_server->gnutls_sess, gnutls_prot_prio);
                 gnutls_credentials_set (session_current_server->gnutls_sess, GNUTLS_CRD_CERTIFICATE, gnutls_xcred);
                 session_data = NULL;
                 rc = rc && (session_read_buf_alloc (file, &session_data, &session_size_int));
@@ -986,6 +987,9 @@ session_load_server (FILE *file)
             case SESSION_SERV_NICK_MODES:
                 rc = rc && (session_read_str (file, &(session_current_server->nick_modes)));
                 break;
+            case SESSION_SERV_PREFIX:
+                rc = rc && (session_read_str (file, &(session_current_server->prefix)));
+                break;
             case SESSION_SERV_RECONNECT_START:
                 rc = rc && (session_read_buf (file, &(session_current_server->reconnect_start), sizeof (time_t)));
                 break;
@@ -1065,9 +1069,9 @@ session_load_channel (FILE *file)
     /* allocate channel */
     weechat_log_printf (_("session: loading channel \"%s\"\n"),
                         channel_name);
-    session_current_channel = channel_new (session_current_server,
-                                           channel_type,
-                                           channel_name);
+    session_current_channel = irc_channel_new (session_current_server,
+                                               channel_type,
+                                               channel_name);
     free (channel_name);
     if (!session_current_channel)
     {
@@ -1160,8 +1164,8 @@ session_load_nick (FILE *file)
     }
     
     /* allocate nick */
-    nick = nick_new (session_current_server, session_current_channel,
-                     nick_name, 0, 0, 0, 0, 0);
+    nick = irc_nick_new (session_current_server, session_current_channel,
+                         nick_name, 0, 0, 0, 0, 0, 0);
     free (nick_name);
     if (!nick)
     {
@@ -1218,7 +1222,7 @@ session_load_dcc (FILE *file)
     t_irc_channel *ptr_channel;
     
     /* allocate DCC */
-    dcc = dcc_alloc ();
+    dcc = irc_dcc_alloc ();
     if (!dcc)
     {
         session_crash (file, _("can't create new DCC"));
@@ -1251,7 +1255,7 @@ session_load_dcc (FILE *file)
                     return 0;
                 if (string && string[0])
                 {
-                    ptr_server = server_search (string);
+                    ptr_server = irc_server_search (string);
                     if (!ptr_server)
                     {
                         session_crash (file, _("server not found for DCC"));
@@ -1272,7 +1276,7 @@ session_load_dcc (FILE *file)
                     return 0;
                 if (string && string[0])
                 {
-                    ptr_channel = channel_search_any (ptr_server, string);
+                    ptr_channel = irc_channel_search_any (ptr_server, string);
                     if (!ptr_channel)
                     {
                         session_crash (file, _("channel not found for DCC"));
@@ -1464,7 +1468,7 @@ session_load_buffer (FILE *file)
     ptr_channel = NULL;
     if (server_name)
     {
-        ptr_server = server_search (server_name);
+        ptr_server = irc_server_search (server_name);
         if (!ptr_server)
         {
             session_crash (file, _("server not found for buffer"));
@@ -1474,7 +1478,7 @@ session_load_buffer (FILE *file)
     
     if (channel_name)
     {
-        ptr_channel = channel_search_any_without_buffer (ptr_server, channel_name);
+        ptr_channel = irc_channel_search_any_without_buffer (ptr_server, channel_name);
         if (!ptr_channel)
         {
             session_crash (file, _("channel not found for buffer"));
@@ -1682,7 +1686,7 @@ session_load_hotlist (FILE *file)
                 server_name = NULL;
                 if (!session_read_str (file, &server_name))
                     return 0;
-                ptr_server = server_search (server_name);
+                ptr_server = irc_server_search (server_name);
                 free (server_name);
                 break;
             case SESSION_HOTL_BUFFER_NUMBER:
