@@ -33,6 +33,14 @@
 #include <iconv.h>
 #endif
 
+#ifndef ICONV_CONST
+  #ifdef ICONV_2ARG_IS_CONST
+    #define ICONV_CONST const
+  #else
+    #define ICONV_CONST
+  #endif
+#endif
+
 #include "weechat.h"
 #include "utf8.h"
 #include "weeconfig.h"
@@ -201,6 +209,7 @@ weechat_iconv (int from_utf8, char *from_code, char *to_code, char *string)
 #ifdef HAVE_ICONV
     iconv_t cd;
     char *inbuf, *ptr_inbuf, *ptr_outbuf, *next_char;
+    char *ptr_inbuf_shift;
     int done;
     size_t err, inbytesleft, outbytesleft;
     
@@ -218,6 +227,7 @@ weechat_iconv (int from_utf8, char *from_code, char *to_code, char *string)
             outbytesleft = inbytesleft * 4;
             outbuf = (char *) malloc (outbytesleft + 2);
             ptr_outbuf = outbuf;
+            ptr_inbuf_shift = NULL;
             done = 0;
             while (!done)
             {
@@ -260,8 +270,19 @@ weechat_iconv (int from_utf8, char *from_code, char *to_code, char *string)
                     }
                 }
                 else
-                    done = 1;
+                {
+                    if (!ptr_inbuf_shift)
+                    {
+                        ptr_inbuf_shift = ptr_inbuf;
+                        ptr_inbuf = NULL;
+                        inbytesleft = 0;
+                    }
+                    else
+                        done = 1;
+                }
             }
+            if (ptr_inbuf_shift)
+                ptr_inbuf = ptr_inbuf_shift;
             ptr_outbuf[0] = '\0';
             free (inbuf);
             iconv_close (cd);
@@ -270,7 +291,7 @@ weechat_iconv (int from_utf8, char *from_code, char *to_code, char *string)
     else
         outbuf = strdup (string);
 #else
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) from_code;
     (void) to_code;
     outbuf = strdup (string);
@@ -298,7 +319,7 @@ weechat_iconv_to_internal (char *charset, char *string)
     
     if (input)
     {
-        if (utf8_is_valid (input, NULL))
+        if (utf8_has_8bits (input) && utf8_is_valid (input, NULL))
             return input;
         
         output = weechat_iconv (0,
@@ -455,33 +476,30 @@ explode_string (char *string, char *separators, int num_items_max,
 
     if (num_items != NULL)
         *num_items = 0;
-
-    n_items = num_items_max;
-
+    
     if (!string || !string[0])
         return NULL;
-
-    if (num_items_max == 0)
+    
+    /* calculate number of items */
+    ptr = string;
+    i = 1;
+    while ((ptr = strpbrk (ptr, separators)))
     {
-        /* calculate number of items */
-        ptr = string;
-        i = 1;
-        while ((ptr = strpbrk (ptr, separators)))
-        {
-            while (strchr (separators, ptr[0]) != NULL)
-                ptr++;
-            i++;
-        }
-        n_items = i;
+        while (strchr (separators, ptr[0]) != NULL)
+            ptr++;
+        i++;
     }
+    n_items = i;
 
+    if ((num_items_max != 0) && (n_items > num_items_max))
+        n_items = num_items_max;
+    
     array =
-        (char **) malloc ((num_items_max ? n_items : n_items + 1) *
-                          sizeof (char *));
-
+        (char **) malloc ((n_items + 1) * sizeof (char *));
+    
     ptr1 = string;
     ptr2 = string;
-
+    
     for (i = 0; i < n_items; i++)
     {
         while (strchr (separators, ptr1[0]) != NULL)
@@ -490,7 +508,7 @@ explode_string (char *string, char *separators, int num_items_max,
             if ((ptr2 = strchr (ptr1, '\r')) == NULL)
                 if ((ptr2 = strchr (ptr1, '\n')) == NULL)
                     ptr2 = strchr (ptr1, '\0');
-
+        
         if ((ptr1 == NULL) || (ptr2 == NULL))
         {
             array[i] = NULL;
@@ -511,18 +529,11 @@ explode_string (char *string, char *separators, int num_items_max,
             }
         }
     }
-    if (num_items_max == 0)
-    {
-        array[i] = NULL;
-        if (num_items != NULL)
-            *num_items = i;
-    }
-    else
-    {
-        if (num_items != NULL)
-            *num_items = num_items_max;
-    }
-
+    
+    array[i] = NULL;
+    if (num_items != NULL)
+        *num_items = i;
+    
     return array;
 }
 
