@@ -78,6 +78,14 @@ gui_input_get_prompt_length (t_gui_window *window, char *nick)
     char *pos, saved_char;
     int char_size, length;
     
+    if (window->buffer->text_search != TEXT_SEARCH_DISABLED)
+    {
+        if (window->buffer->text_search_exact)
+            return utf8_width_screen (_("Text search (exact): "));
+        else
+            return utf8_width_screen (_("Text search: "));
+    }
+    
     length = 0;
     pos = cfg_look_input_format;
     while (pos && pos[0])
@@ -88,7 +96,7 @@ gui_input_get_prompt_length (t_gui_window *window, char *nick)
                 pos++;
                 switch (pos[0])
                 {
-                    case 'c':
+                    case 'c': /* channel or server name */
                         if (CHANNEL(window->buffer))
                             length += utf8_width_screen (CHANNEL(window->buffer)->name);
                         else
@@ -98,8 +106,8 @@ gui_input_get_prompt_length (t_gui_window *window, char *nick)
                         }
                         pos++;
                         break;
-                    case 'm':
-                        if (SERVER(window->buffer))
+                    case 'm': /* nick modes */
+                        if (SERVER(window->buffer) && SERVER(window->buffer)->is_connected)
                         {
                             if (SERVER(window->buffer)->nick_modes
                                 && SERVER(window->buffer)->nick_modes[0])
@@ -107,7 +115,7 @@ gui_input_get_prompt_length (t_gui_window *window, char *nick)
                         }
                         pos++;
                         break;
-                    case 'n':
+                    case 'n': /* nick */
                         length += utf8_width_screen (nick);
                         pos++;
                         break;
@@ -152,6 +160,17 @@ gui_input_draw_prompt (t_gui_window *window, char *nick)
     int char_size;
     
     wmove (GUI_CURSES(window)->win_input, 0, 0);
+    
+    if (window->buffer->text_search != TEXT_SEARCH_DISABLED)
+    {
+        gui_window_set_weechat_color (GUI_CURSES(window)->win_input,
+                                      COLOR_WIN_INPUT);
+        wprintw (GUI_CURSES(window)->win_input, "%s",
+                 (window->buffer->text_search_exact) ?
+                 _("Text search (exact): ") : _("Text search: "));
+        return;
+    }
+    
     pos = cfg_look_input_format;
     while (pos && pos[0])
     {
@@ -161,7 +180,7 @@ gui_input_draw_prompt (t_gui_window *window, char *nick)
                 pos++;
                 switch (pos[0])
                 {
-                    case 'c':
+                    case 'c': /* channel or server name */
                         if (CHANNEL(window->buffer))
                         {
                             gui_window_set_weechat_color (GUI_CURSES(window)->win_input,
@@ -189,8 +208,8 @@ gui_input_draw_prompt (t_gui_window *window, char *nick)
                         }
                         pos++;
                         break;
-                    case 'm':
-                        if (SERVER(window->buffer))
+                    case 'm': /* nick modes */
+                        if (SERVER(window->buffer) && SERVER(window->buffer)->is_connected)
                         {
                             if (SERVER(window->buffer)->nick_modes
                                 && SERVER(window->buffer)->nick_modes[0])
@@ -203,7 +222,7 @@ gui_input_draw_prompt (t_gui_window *window, char *nick)
                         }
                         pos++;
                         break;
-                    case 'n':
+                    case 'n': /* nick */
                         gui_window_set_weechat_color (GUI_CURSES(window)->win_input,
                                                       COLOR_WIN_INPUT_NICK);
                         buf = weechat_iconv_from_internal (NULL, nick);
@@ -265,6 +284,15 @@ gui_input_draw_text (t_gui_window *window, int input_width)
     count_cursor = window->buffer->input_buffer_pos -
         window->buffer->input_buffer_1st_display;
     offset_cursor = 0;
+    if (window->buffer->text_search != TEXT_SEARCH_DISABLED)
+    {
+        if (window->buffer->text_search_found)
+            gui_window_set_weechat_color (GUI_CURSES(window)->win_input,
+                                          COLOR_WIN_INPUT);
+        else
+            gui_window_set_weechat_color (GUI_CURSES(window)->win_input,
+                                          COLOR_WIN_INPUT_TEXT_NOT_FOUND);
+    }
     while ((input_width > 0) && ptr_start && ptr_start[0])
     {
         ptr_next = utf8_next_char (ptr_start);
@@ -273,18 +301,21 @@ gui_input_draw_text (t_gui_window *window, int input_width)
             saved_char = ptr_next[0];
             ptr_next[0] = '\0';
             size = ptr_next - ptr_start;
-            if (window->buffer->input_buffer_color_mask[pos_mask] != ' ')
-                color = window->buffer->input_buffer_color_mask[pos_mask] - '0';
-            else
-                color = -1;
-            if (color != last_color)
+            if (window->buffer->text_search == TEXT_SEARCH_DISABLED)
             {
-                if (color == -1)
-                    gui_window_set_weechat_color (GUI_CURSES(window)->win_input, COLOR_WIN_INPUT);
+                if (window->buffer->input_buffer_color_mask[pos_mask] != ' ')
+                    color = window->buffer->input_buffer_color_mask[pos_mask] - '0';
                 else
-                    gui_input_set_color (window, color);
+                    color = -1;
+                if (color != last_color)
+                {
+                    if (color == -1)
+                        gui_window_set_weechat_color (GUI_CURSES(window)->win_input, COLOR_WIN_INPUT);
+                    else
+                        gui_input_set_color (window, color);
+                }
+                last_color = color;
             }
-            last_color = color;
             output = weechat_iconv_from_internal (NULL, ptr_start);
             wprintw (GUI_CURSES(window)->win_input, "%s", (output) ? output : ptr_start);
             if (output)
@@ -336,9 +367,9 @@ gui_input_draw (t_gui_buffer *buffer, int erase)
                         if (buffer->input_buffer_length == 0)
                             buffer->input_buffer[0] = '\0';
                         
-                        if (SERVER(buffer))
+                        if (SERVER(buffer) && SERVER(buffer)->is_connected)
                             ptr_nickname = (SERVER(buffer)->nick) ?
-                                SERVER(buffer)->nick : SERVER(buffer)->nick1;
+                                SERVER(buffer)->nick : cfg_look_no_nickname;
                         else
                             ptr_nickname = cfg_look_no_nickname;
                         
