@@ -48,7 +48,7 @@ gui_status_draw (t_gui_buffer *buffer, int erase)
     int x, server_pos, server_total;
     int display_name, names_count;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) buffer;
     
     if (!gui_ok)
@@ -110,16 +110,26 @@ gui_status_draw (t_gui_buffer *buffer, int erase)
             gui_window_set_weechat_color (GUI_CURSES(ptr_win)->win_status,
                                           COLOR_WIN_STATUS_CHANNEL);
             if (SERVER(ptr_win->buffer)->is_connected)
-                wprintw (GUI_CURSES(ptr_win)->win_status, "[%s] ",
-                         SERVER(ptr_win->buffer)->name);
+            {
+                if ((ptr_win->buffer->all_servers) && (SERVER(ptr_win->buffer)->is_away))
+                {
+                    wprintw (GUI_CURSES(ptr_win)->win_status, "[%s",
+                             SERVER(ptr_win->buffer)->name);
+                    wprintw (GUI_CURSES(ptr_win)->win_status, _("(away)"));
+                    wprintw (GUI_CURSES(ptr_win)->win_status, "] ");
+                }
+                else
+                    wprintw (GUI_CURSES(ptr_win)->win_status, "[%s] ",
+                             SERVER(ptr_win->buffer)->name);
+            }
             else
                 wprintw (GUI_CURSES(ptr_win)->win_status, "(%s) ",
                          SERVER(ptr_win->buffer)->name);
             if (ptr_win->buffer->all_servers)
             {
-                server_get_number_buffer (SERVER(ptr_win->buffer),
-                                          &server_pos,
-                                          &server_total);
+                irc_server_get_number_buffer (SERVER(ptr_win->buffer),
+                                              &server_pos,
+                                              &server_total);
                 gui_window_set_weechat_color (GUI_CURSES(ptr_win)->win_status,
                                               COLOR_WIN_STATUS_DELIMITERS);
                 wprintw (GUI_CURSES(ptr_win)->win_status, "(");
