@@ -206,7 +206,7 @@ gui_window_calculate_pos_size (t_gui_window *window, int force_calculate)
     /* init chat & nicklist settings */
     if (cfg_look_nicklist && BUFFER_IS_CHANNEL(window->buffer))
     {
-        max_length = nick_get_max_length (CHANNEL(window->buffer));
+        max_length = irc_nick_get_max_length (CHANNEL(window->buffer));
         
         lines = 0;
         
@@ -224,8 +224,8 @@ gui_window_calculate_pos_size (t_gui_window *window, int force_calculate)
         }
         else
         {
-            nick_count (CHANNEL(window->buffer), &num_nicks, &num_op,
-                        &num_halfop, &num_voice, &num_normal);
+            irc_nick_count (CHANNEL(window->buffer), &num_nicks, &num_op,
+                            &num_halfop, &num_voice, &num_normal);
             width_used = (window->win_width - add_left - add_right)
                 - ((window->win_width - add_left - add_right) % (max_length + 2));
             if (((max_length + 2) * num_nicks) % width_used == 0)
@@ -533,7 +533,12 @@ gui_window_page_up (t_gui_window *window)
                                       (-1) * (window->win_chat_height - 1) :
                                       (-1) * ((window->win_chat_height - 1) * 2));
         gui_chat_draw (window->buffer, 0);
-        gui_status_draw (window->buffer, 0);
+        if (!window->scroll)
+        {
+            window->start_line = NULL;
+            window->start_line_pos = 0;
+        }
+        gui_status_draw (window->buffer, 1);
     }
 }
 
@@ -569,7 +574,13 @@ gui_window_page_down (t_gui_window *window)
         }
         
         gui_chat_draw (window->buffer, 0);
-        gui_status_draw (window->buffer, 0);
+        if (!window->scroll)
+        {
+            window->start_line = NULL;
+            window->start_line_pos = 0;
+            hotlist_remove_buffer (window->buffer);
+        }
+        gui_status_draw (window->buffer, 1);
     }
 }
 
@@ -591,7 +602,12 @@ gui_window_scroll_up (t_gui_window *window)
                                       (-1) * cfg_look_scroll_amount :
                                       (-1) * ( (window->win_chat_height - 1) + cfg_look_scroll_amount));
         gui_chat_draw (window->buffer, 0);
-        gui_status_draw (window->buffer, 0);
+        if (!window->scroll)
+        {
+            window->start_line = NULL;
+            window->start_line_pos = 0;
+        }
+        gui_status_draw (window->buffer, 1);
     }
 }
 
@@ -628,7 +644,13 @@ gui_window_scroll_down (t_gui_window *window)
         }
         
         gui_chat_draw (window->buffer, 0);
-        gui_status_draw (window->buffer, 0);
+        if (!window->scroll)
+        {
+            window->start_line = NULL;
+            window->start_line_pos = 0;
+            hotlist_remove_buffer (window->buffer);
+        }
+        gui_status_draw (window->buffer, 1);
     }
 }
 
@@ -647,7 +669,12 @@ gui_window_scroll_top (t_gui_window *window)
         window->start_line = window->buffer->lines;
         window->start_line_pos = 0;
         gui_chat_draw (window->buffer, 0);
-        gui_status_draw (window->buffer, 0);
+        if (!window->scroll)
+        {
+            window->start_line = NULL;
+            window->start_line_pos = 0;
+        }
+        gui_status_draw (window->buffer, 1);
     }
 }
 
@@ -666,7 +693,13 @@ gui_window_scroll_bottom (t_gui_window *window)
         window->start_line = NULL;
         window->start_line_pos = 0;
         gui_chat_draw (window->buffer, 0);
-        gui_status_draw (window->buffer, 0);
+        if (!window->scroll)
+        {
+            window->start_line = NULL;
+            window->start_line_pos = 0;
+            hotlist_remove_buffer (window->buffer);
+        }
+        gui_status_draw (window->buffer, 1);
     }
 }
 
