@@ -70,14 +70,16 @@ gui_action_clipboard_copy (char *buffer, int size)
  */
 
 void
-gui_action_clipboard_paste (t_gui_window *window)
+gui_action_clipboard_paste (t_gui_window *window, char *args)
 {
+    /* make C compiler happy */
+    (void) args;
+    
     if (window->buffer->has_input && gui_input_clipboard) 
     {
         gui_insert_string_input (window, gui_input_clipboard, -1);
-        window->buffer->input_buffer_pos += utf8_strlen (gui_input_clipboard);
-        gui_input_draw (window->buffer, 0);
         window->buffer->completion.position = -1;
+        gui_input_draw (window->buffer, 0);
     }
 }
 
@@ -86,13 +88,18 @@ gui_action_clipboard_paste (t_gui_window *window)
  */
 
 void
-gui_action_return (t_gui_window *window)
+gui_action_return (t_gui_window *window, char *args)
 {
     char *command;
     
+    /* make C compiler happy */
+    (void) args;
+    
     if (window->buffer->has_input)
     {
-        if (window->buffer->input_buffer_size > 0)
+        if (window->buffer->text_search != TEXT_SEARCH_DISABLED)
+            gui_buffer_search_stop (window);
+        else if (window->buffer->input_buffer_size > 0)
         {
             window->buffer->input_buffer[window->buffer->input_buffer_size] = '\0';
             window->buffer->input_buffer_color_mask[window->buffer->input_buffer_size] = '\0';
@@ -123,9 +130,13 @@ gui_action_return (t_gui_window *window)
  */
 
 void
-gui_action_tab (t_gui_window *window)
+gui_action_tab (t_gui_window *window, char *args)
 {
-    if (window->buffer->has_input)
+    /* make C compiler happy */
+    (void) args;
+    
+    if (window->buffer->has_input
+        && (window->buffer->text_search == TEXT_SEARCH_DISABLED))
     {
         completion_search (&(window->buffer->completion), 1,
                            window->buffer->input_buffer,
@@ -141,9 +152,13 @@ gui_action_tab (t_gui_window *window)
  */
 
 void
-gui_action_tab_previous (t_gui_window *window)
+gui_action_tab_previous (t_gui_window *window, char *args)
 {
-    if (window->buffer->has_input)
+    /* make C compiler happy */
+    (void) args;
+    
+    if (window->buffer->has_input
+        && (window->buffer->text_search == TEXT_SEARCH_DISABLED))
     {
         completion_search (&(window->buffer->completion), -1,
                            window->buffer->input_buffer,
@@ -159,11 +174,14 @@ gui_action_tab_previous (t_gui_window *window)
  */
 
 void
-gui_action_backspace (t_gui_window *window)
+gui_action_backspace (t_gui_window *window, char *args)
 {
     char *pos, *pos_last;
     int char_size, size_to_move;
-
+    
+    /* make C compiler happy */
+    (void) args;
+    
     if (window->buffer->has_input)
     {
         if (window->buffer->input_buffer_pos > 0)
@@ -187,14 +205,17 @@ gui_action_backspace (t_gui_window *window)
 }
 
 /*
- * gui_action_delete: delete key
+ * gui_action_delete: delete next char
  */
 
 void
-gui_action_delete (t_gui_window *window)
+gui_action_delete (t_gui_window *window, char *args)
 {
     char *pos, *pos_next;
     int char_size, size_to_move;
+
+    /* make C compiler happy */
+    (void) args;
     
     if (window->buffer->has_input)
     {
@@ -223,11 +244,14 @@ gui_action_delete (t_gui_window *window)
  */
 
 void
-gui_action_delete_previous_word (t_gui_window *window)
+gui_action_delete_previous_word (t_gui_window *window, char *args)
 {
     int length_deleted, size_deleted;
     char *start, *string;
     
+    /* make C compiler happy */
+    (void) args;
+    
     if (window->buffer->has_input)
     {
         if (window->buffer->input_buffer_pos > 0)
@@ -283,11 +307,14 @@ gui_action_delete_previous_word (t_gui_window *window)
  */
 
 void
-gui_action_delete_next_word (t_gui_window *window)
+gui_action_delete_next_word (t_gui_window *window, char *args)
 {
     int size_deleted, length_deleted;
     char *start, *string;
     
+    /* make C compiler happy */
+    (void) args;
+    
     if (window->buffer->has_input)
     {
         start = utf8_add_offset (window->buffer->input_buffer,
@@ -322,10 +349,13 @@ gui_action_delete_next_word (t_gui_window *window)
  */
 
 void
-gui_action_delete_begin_of_line (t_gui_window *window)
+gui_action_delete_begin_of_line (t_gui_window *window, char *args)
 {
     int length_deleted, size_deleted, pos_start;
     char *start;
+
+    /* make C compiler happy */
+    (void) args;
     
     if (window->buffer->has_input)
     {
@@ -358,10 +388,13 @@ gui_action_delete_begin_of_line (t_gui_window *window)
  */
 
 void
-gui_action_delete_end_of_line (t_gui_window *window)
+gui_action_delete_end_of_line (t_gui_window *window, char *args)
 {
     char *start;
     int size_deleted, length_deleted, pos_start;
+
+    /* make C compiler happy */
+    (void) args;
     
     if (window->buffer->has_input)
     {
@@ -386,8 +419,11 @@ gui_action_delete_end_of_line (t_gui_window *window)
  */
 
 void
-gui_action_delete_line (t_gui_window *window)
+gui_action_delete_line (t_gui_window *window, char *args)
 {
+    /* make C compiler happy */
+    (void) args;
+    
     if (window->buffer->has_input)
     {
         window->buffer->input_buffer[0] = '\0';
@@ -406,8 +442,11 @@ gui_action_delete_line (t_gui_window *window)
  */
 
 void
-gui_action_transpose_chars (t_gui_window *window)
+gui_action_transpose_chars (t_gui_window *window, char *args)
 {
+    /* make C compiler happy */
+    (void) args;
+    
     char *start, *prev_char, saved_char[5];
     int size_prev_char, size_start_char;
     int pos_prev_char, pos_start;
@@ -452,8 +491,11 @@ gui_action_transpose_chars (t_gui_window *window)
  */
 
 void
-gui_action_home (t_gui_window *window)
+gui_action_home (t_gui_window *window, char *args)
 {
+    /* make C compiler happy */
+    (void) args;
+    
     if (window->buffer->has_input)
     {
         if (window->buffer->input_buffer_pos > 0)
@@ -469,8 +511,11 @@ gui_action_home (t_gui_window *window)
  */
 
 void
-gui_action_end (t_gui_window *window)
+gui_action_end (t_gui_window *window, char *args)
 {
+    /* make C compiler happy */
+    (void) args;
+    
     if (window->buffer->has_input)
     {
         if (window->buffer->input_buffer_pos <
@@ -488,8 +533,11 @@ gui_action_end (t_gui_window *window)
  */
 
 void
-gui_action_left (t_gui_window *window)
+gui_action_left (t_gui_window *window, char *args)
 {
+    /* make C compiler happy */
+    (void) args;
+    
     if (window->buffer->has_input)
     {
         if (window->buffer->input_buffer_pos > 0)
@@ -505,10 +553,13 @@ gui_action_left (t_gui_window *window)
  */
 
 void
-gui_action_previous_word (t_gui_window *window)
+gui_action_previous_word (t_gui_window *window, char *args)
 {
     char *pos;
     
+    /* make C compiler happy */
+    (void) args;
+    
     if (window->buffer->has_input)
     {
         if (window->buffer->input_buffer_pos > 0)
@@ -545,8 +596,11 @@ gui_action_previous_word (t_gui_window *window)
  */
 
 void
-gui_action_right (t_gui_window *window)
+gui_action_right (t_gui_window *window, char *args)
 {
+    /* make C compiler happy */
+    (void) args;
+    
     if (window->buffer->has_input)
     {
         if (window->buffer->input_buffer_pos <
@@ -563,10 +617,13 @@ gui_action_right (t_gui_window *window)
  */
 
 void
-gui_action_next_word (t_gui_window *window)
+gui_action_next_word (t_gui_window *window, char *args)
 {
     char *pos;
     
+    /* make C compiler happy */
+    (void) args;
+    
     if (window->buffer->has_input)
     {
         if (window->buffer->input_buffer_pos <
@@ -607,8 +664,11 @@ gui_action_next_word (t_gui_window *window)
  */
 
 void
-gui_action_up (t_gui_window *window)
+gui_action_up (t_gui_window *window, char *args)
 {
+    /* make C compiler happy */
+    (void) args;
+    
     if (window->buffer->type == BUFFER_TYPE_DCC)
     {
         if (dcc_list)
@@ -629,53 +689,62 @@ gui_action_up (t_gui_window *window)
     }
     else if (window->buffer->has_input)
     {
-        if (window->buffer->ptr_history)
+        if (window->buffer->text_search == TEXT_SEARCH_DISABLED)
         {
-            window->buffer->ptr_history =
-                window->buffer->ptr_history->next_history;
-            if (!window->buffer->ptr_history)
-                window->buffer->ptr_history =
-                    window->buffer->history;
-        }
-        else
-            window->buffer->ptr_history =
-                window->buffer->history;
-        if (window->buffer->ptr_history)
-        {
-	    /* bash/readline like use of history */
-            if (window->buffer->ptr_history->prev_history == NULL)
+            if (window->buffer->ptr_history)
             {
-                if (window->buffer->input_buffer_size > 0)
-                {
-                    window->buffer->input_buffer[window->buffer->input_buffer_size] = '\0';
-                    window->buffer->input_buffer_color_mask[window->buffer->input_buffer_size] = '\0';
-                    history_buffer_add (window->buffer, window->buffer->input_buffer);
-                    history_global_add (window->buffer->input_buffer);
-                }
+                window->buffer->ptr_history =
+                    window->buffer->ptr_history->next_history;
+                if (!window->buffer->ptr_history)
+                    window->buffer->ptr_history =
+                        window->buffer->history;
             }
             else
+                window->buffer->ptr_history =
+                    window->buffer->history;
+            if (window->buffer->ptr_history)
             {
-                if (window->buffer->input_buffer_size > 0)
+                /* bash/readline like use of history */
+                if (window->buffer->ptr_history->prev_history == NULL)
                 {
-                    window->buffer->input_buffer[window->buffer->input_buffer_size] = '\0';
-                    window->buffer->input_buffer_color_mask[window->buffer->input_buffer_size] = '\0';
-                    if (window->buffer->ptr_history->prev_history->text)
-			free(window->buffer->ptr_history->prev_history->text);
-                    window->buffer->ptr_history->prev_history->text = strdup (window->buffer->input_buffer);
+                    if (window->buffer->input_buffer_size > 0)
+                    {
+                        window->buffer->input_buffer[window->buffer->input_buffer_size] = '\0';
+                        window->buffer->input_buffer_color_mask[window->buffer->input_buffer_size] = '\0';
+                        history_buffer_add (window->buffer, window->buffer->input_buffer);
+                        history_global_add (window->buffer->input_buffer);
+                    }
                 }
+                else
+                {
+                    if (window->buffer->input_buffer_size > 0)
+                    {
+                        window->buffer->input_buffer[window->buffer->input_buffer_size] = '\0';
+                        window->buffer->input_buffer_color_mask[window->buffer->input_buffer_size] = '\0';
+                        if (window->buffer->ptr_history->prev_history->text)
+                            free(window->buffer->ptr_history->prev_history->text);
+                        window->buffer->ptr_history->prev_history->text = strdup (window->buffer->input_buffer);
+                    }
+                }
+                window->buffer->input_buffer_size =
+                    strlen (window->buffer->ptr_history->text);
+                window->buffer->input_buffer_length =
+                    utf8_strlen (window->buffer->ptr_history->text);
+                gui_input_optimize_size (window->buffer);
+                window->buffer->input_buffer_pos =
+                    window->buffer->input_buffer_length;
+                window->buffer->input_buffer_1st_display = 0;
+                strcpy (window->buffer->input_buffer,
+                        window->buffer->ptr_history->text);
+                gui_input_init_color_mask (window->buffer);
+                gui_input_draw (window->buffer, 0);
             }
-            window->buffer->input_buffer_size =
-                strlen (window->buffer->ptr_history->text);
-            window->buffer->input_buffer_length =
-                utf8_strlen (window->buffer->ptr_history->text);
-            gui_input_optimize_size (window->buffer);
-            window->buffer->input_buffer_pos =
-                window->buffer->input_buffer_length;
-            window->buffer->input_buffer_1st_display = 0;
-            strcpy (window->buffer->input_buffer,
-                    window->buffer->ptr_history->text);
-            gui_input_init_color_mask (window->buffer);
-            gui_input_draw (window->buffer, 0);
+        }
+        else
+        {
+            /* search backward in buffer history */
+            window->buffer->text_search = TEXT_SEARCH_BACKWARD;
+            (void) gui_buffer_search_text (window);
         }
     }
 }
@@ -685,9 +754,13 @@ gui_action_up (t_gui_window *window)
  */
 
 void
-gui_action_up_global (t_gui_window *window)
+gui_action_up_global (t_gui_window *window, char *args)
 {
-    if (window->buffer->has_input)
+    /* make C compiler happy */
+    (void) args;
+    
+    if (window->buffer->has_input
+        && (window->buffer->text_search == TEXT_SEARCH_DISABLED))
     {
         if (history_global_ptr)
         {
@@ -720,8 +793,11 @@ gui_action_up_global (t_gui_window *window)
  */
 
 void
-gui_action_down (t_gui_window *window)
+gui_action_down (t_gui_window *window, char *args)
 {
+    /* make C compiler happy */
+    (void) args;
+    
     if (window->buffer->type == BUFFER_TYPE_DCC)
     {
         if (dcc_list)
@@ -753,33 +829,42 @@ gui_action_down (t_gui_window *window)
     }
     else if (window->buffer->has_input)
     {
-        if (window->buffer->ptr_history)
+        if (window->buffer->text_search == TEXT_SEARCH_DISABLED)
         {
-            window->buffer->ptr_history =
-                window->buffer->ptr_history->prev_history;
             if (window->buffer->ptr_history)
             {
-                window->buffer->input_buffer_size =
-                    strlen (window->buffer->ptr_history->text);
-                window->buffer->input_buffer_length =
-                    utf8_strlen (window->buffer->ptr_history->text);
+                window->buffer->ptr_history =
+                    window->buffer->ptr_history->prev_history;
+                if (window->buffer->ptr_history)
+                {
+                    window->buffer->input_buffer_size =
+                        strlen (window->buffer->ptr_history->text);
+                    window->buffer->input_buffer_length =
+                        utf8_strlen (window->buffer->ptr_history->text);
+                }
+                else
+                {
+                    window->buffer->input_buffer_size = 0;
+                    window->buffer->input_buffer_length = 0;
+                }
+                gui_input_optimize_size (window->buffer);
+                window->buffer->input_buffer_pos =
+                    window->buffer->input_buffer_length;
+                window->buffer->input_buffer_1st_display = 0;
+                if (window->buffer->ptr_history)
+                {
+                    strcpy (window->buffer->input_buffer,
+                            window->buffer->ptr_history->text);
+                    gui_input_init_color_mask (window->buffer);
+                }
+                gui_input_draw (window->buffer, 0);
             }
-            else
-            {
-                window->buffer->input_buffer_size = 0;
-                window->buffer->input_buffer_length = 0;
-            }
-            gui_input_optimize_size (window->buffer);
-            window->buffer->input_buffer_pos =
-                window->buffer->input_buffer_length;
-            window->buffer->input_buffer_1st_display = 0;
-            if (window->buffer->ptr_history)
-            {
-                strcpy (window->buffer->input_buffer,
-                        window->buffer->ptr_history->text);
-                gui_input_init_color_mask (window->buffer);
-            }
-            gui_input_draw (window->buffer, 0);
+        }
+        else
+        {
+            /* search forward in buffer history */
+            window->buffer->text_search = TEXT_SEARCH_FORWARD;
+            (void) gui_buffer_search_text (window);
         }
     }
 }
@@ -789,9 +874,13 @@ gui_action_down (t_gui_window *window)
  */
 
 void
-gui_action_down_global (t_gui_window *window)
+gui_action_down_global (t_gui_window *window, char *args)
 {
-    if (window->buffer->has_input)
+    /* make C compiler happy */
+    (void) args;
+    
+    if (window->buffer->has_input
+        && (window->buffer->text_search == TEXT_SEARCH_DISABLED))
     {
         if (history_global_ptr)
         {
@@ -828,8 +917,11 @@ gui_action_down_global (t_gui_window *window)
  */
 
 void
-gui_action_page_up (t_gui_window *window)
+gui_action_page_up (t_gui_window *window, char *args)
 {
+    /* make C compiler happy */
+    (void) args;
+    
     gui_window_page_up (window);
 }
 
@@ -838,8 +930,11 @@ gui_action_page_up (t_gui_window *window)
  */
 
 void
-gui_action_page_down (t_gui_window *window)
+gui_action_page_down (t_gui_window *window, char *args)
 {
+    /* make C compiler happy */
+    (void) args;
+    
     gui_window_page_down (window);
 }
 
@@ -848,8 +943,11 @@ gui_action_page_down (t_gui_window *window)
  */
 
 void
-gui_action_scroll_up (t_gui_window *window)
+gui_action_scroll_up (t_gui_window *window, char *args)
 {
+    /* make C compiler happy */
+    (void) args;
+    
     gui_window_scroll_up (window);
 }
 
@@ -858,8 +956,11 @@ gui_action_scroll_up (t_gui_window *window)
  */
 
 void
-gui_action_scroll_down (t_gui_window *window)
+gui_action_scroll_down (t_gui_window *window, char *args)
 {
+    /* make C compiler happy */
+    (void) args;
+    
     gui_window_scroll_down (window);
 }
 
@@ -868,8 +969,11 @@ gui_action_scroll_down (t_gui_window *window)
  */
 
 void
-gui_action_scroll_top (t_gui_window *window)
+gui_action_scroll_top (t_gui_window *window, char *args)
 {
+    /* make C compiler happy */
+    (void) args;
+    
     gui_window_scroll_top (window);
 }
 
@@ -878,8 +982,11 @@ gui_action_scroll_top (t_gui_window *window)
  */
 
 void
-gui_action_scroll_bottom (t_gui_window *window)
+gui_action_scroll_bottom (t_gui_window *window, char *args)
 {
+    /* make C compiler happy */
+    (void) args;
+    
     gui_window_scroll_bottom (window);
 }
 
@@ -888,8 +995,11 @@ gui_action_scroll_bottom (t_gui_window *window)
  */
 
 void
-gui_action_scroll_topic_left (t_gui_window *window)
+gui_action_scroll_topic_left (t_gui_window *window, char *args)
 {
+    /* make C compiler happy */
+    (void) args;
+    
     gui_window_scroll_topic_left (window);
 }
 
@@ -898,8 +1008,11 @@ gui_action_scroll_topic_left (t_gui_window *window)
  */
 
 void
-gui_action_scroll_topic_right (t_gui_window *window)
+gui_action_scroll_topic_right (t_gui_window *window, char *args)
 {
+    /* make C compiler happy */
+    (void) args;
+    
     gui_window_scroll_topic_right (window);
 }
 
@@ -908,8 +1021,11 @@ gui_action_scroll_topic_right (t_gui_window *window)
  */
 
 void
-gui_action_nick_beginning (t_gui_window *window)
+gui_action_nick_beginning (t_gui_window *window, char *args)
 {
+    /* make C compiler happy */
+    (void) args;
+    
     gui_window_nick_beginning (window);
 }
 
@@ -918,8 +1034,11 @@ gui_action_nick_beginning (t_gui_window *window)
  */
 
 void
-gui_action_nick_end (t_gui_window *window)
+gui_action_nick_end (t_gui_window *window, char *args)
 {
+    /* make C compiler happy */
+    (void) args;
+    
     gui_window_nick_end (window);
 }
 
@@ -928,8 +1047,11 @@ gui_action_nick_end (t_gui_window *window)
  */
 
 void
-gui_action_nick_page_up (t_gui_window *window)
+gui_action_nick_page_up (t_gui_window *window, char *args)
 {
+    /* make C compiler happy */
+    (void) args;
+    
     gui_window_nick_page_up (window);
 }
 
@@ -938,8 +1060,11 @@ gui_action_nick_page_up (t_gui_window *window)
  */
 
 void
-gui_action_nick_page_down (t_gui_window *window)
+gui_action_nick_page_down (t_gui_window *window, char *args)
 {
+    /* make C compiler happy */
+    (void) args;
+    
     gui_window_nick_page_down (window);
 }
 
@@ -948,22 +1073,28 @@ gui_action_nick_page_down (t_gui_window *window)
  */
 
 void
-gui_action_jump_smart (t_gui_window *window)
+gui_action_jump_smart (t_gui_window *window, char *args)
 {
-    if (hotlist)
+    /* make C compiler happy */
+    (void) args;
+
+    if (window->buffer->text_search == TEXT_SEARCH_DISABLED)
     {
-        if (!hotlist_initial_buffer)
-            hotlist_initial_buffer = window->buffer;
-        gui_window_switch_to_buffer (window, hotlist->buffer);
-        gui_window_redraw_buffer (window->buffer);
-    }
-    else
-    {
-        if (hotlist_initial_buffer)
+        if (hotlist)
         {
-            gui_window_switch_to_buffer (window, hotlist_initial_buffer);
+            if (!hotlist_initial_buffer)
+                hotlist_initial_buffer = window->buffer;
+            gui_window_switch_to_buffer (window, hotlist->buffer);
             gui_window_redraw_buffer (window->buffer);
-            hotlist_initial_buffer = NULL;
+        }
+        else
+        {
+            if (hotlist_initial_buffer)
+            {
+                gui_window_switch_to_buffer (window, hotlist_initial_buffer);
+                gui_window_redraw_buffer (window->buffer);
+                hotlist_initial_buffer = NULL;
+            }
         }
     }
 }
@@ -973,21 +1104,27 @@ gui_action_jump_smart (t_gui_window *window)
  */
 
 void
-gui_action_jump_dcc (t_gui_window *window)
+gui_action_jump_dcc (t_gui_window *window, char *args)
 {
-    if (window->buffer->type == BUFFER_TYPE_DCC)
+    /* make C compiler happy */
+    (void) args;
+
+    if (window->buffer->text_search == TEXT_SEARCH_DISABLED)
     {
-        if (gui_buffer_before_dcc)
+        if (window->buffer->type == BUFFER_TYPE_DCC)
         {
-            gui_window_switch_to_buffer (window,
-                                         gui_buffer_before_dcc);
-            gui_window_redraw_buffer (window->buffer);
+            if (gui_buffer_before_dcc)
+            {
+                gui_window_switch_to_buffer (window,
+                                             gui_buffer_before_dcc);
+                gui_window_redraw_buffer (window->buffer);
+            }
+        }
+        else
+        {
+            gui_buffer_before_dcc = window->buffer;
+            gui_buffer_switch_dcc (window);
         }
-    }
-    else
-    {
-        gui_buffer_before_dcc = window->buffer;
-        gui_buffer_switch_dcc (window);
     }
 }
 
@@ -996,21 +1133,27 @@ gui_action_jump_dcc (t_gui_window *window)
  */
 
 void
-gui_action_jump_raw_data (t_gui_window *window)
+gui_action_jump_raw_data (t_gui_window *window, char *args)
 {
-    if (window->buffer->type == BUFFER_TYPE_RAW_DATA)
+    /* make C compiler happy */
+    (void) args;
+
+    if (window->buffer->text_search == TEXT_SEARCH_DISABLED)
     {
-        if (gui_buffer_before_raw_data)
+        if (window->buffer->type == BUFFER_TYPE_RAW_DATA)
         {
-            gui_window_switch_to_buffer (window,
-                                         gui_buffer_before_raw_data);
-            gui_window_redraw_buffer (window->buffer);
+            if (gui_buffer_before_raw_data)
+            {
+                gui_window_switch_to_buffer (window,
+                                             gui_buffer_before_raw_data);
+                gui_window_redraw_buffer (window->buffer);
+            }
+        }
+        else
+        {
+            gui_buffer_before_raw_data = window->buffer;
+            gui_buffer_switch_raw_data (window);
         }
-    }
-    else
-    {
-        gui_buffer_before_raw_data = window->buffer;
-        gui_buffer_switch_raw_data (window);
     }
 }
 
@@ -1019,10 +1162,16 @@ gui_action_jump_raw_data (t_gui_window *window)
  */
 
 void
-gui_action_jump_last_buffer (t_gui_window *window)
+gui_action_jump_last_buffer (t_gui_window *window, char *args)
 {
-    if (last_gui_buffer)
-        gui_buffer_switch_by_number (window, last_gui_buffer->number);
+    /* make C compiler happy */
+    (void) args;
+
+    if (window->buffer->text_search == TEXT_SEARCH_DISABLED)
+    {
+        if (last_gui_buffer)
+            gui_buffer_switch_by_number (window, last_gui_buffer->number);
+    }
 }
 
 /*
@@ -1030,16 +1179,22 @@ gui_action_jump_last_buffer (t_gui_window *window)
  */
 
 void
-gui_action_jump_server (t_gui_window *window)
+gui_action_jump_server (t_gui_window *window, char *args)
 {
-    if (SERVER(window->buffer))
+    /* make C compiler happy */
+    (void) args;
+    
+    if (window->buffer->text_search == TEXT_SEARCH_DISABLED)
     {
-        if (SERVER(window->buffer)->buffer !=
-            window->buffer)
+        if (SERVER(window->buffer))
         {
-            gui_window_switch_to_buffer (window,
-                                         SERVER(window->buffer)->buffer);
-            gui_window_redraw_buffer (window->buffer);
+            if (SERVER(window->buffer)->buffer !=
+                window->buffer)
+            {
+                gui_window_switch_to_buffer (window,
+                                             SERVER(window->buffer)->buffer);
+                gui_window_redraw_buffer (window->buffer);
+            }
         }
     }
 }
@@ -1049,39 +1204,45 @@ gui_action_jump_server (t_gui_window *window)
  */
 
 void
-gui_action_jump_next_server (t_gui_window *window)
+gui_action_jump_next_server (t_gui_window *window, char *args)
 {
     t_irc_server *ptr_server;
     t_gui_buffer *ptr_buffer;
     
-    if (SERVER(window->buffer))
+    /* make C compiler happy */
+    (void) args;
+    
+    if (window->buffer->text_search == TEXT_SEARCH_DISABLED)
     {
-        ptr_server = SERVER(window->buffer)->next_server;
-        if (!ptr_server)
-            ptr_server = irc_servers;
-        while (ptr_server != SERVER(window->buffer))
+        if (SERVER(window->buffer))
         {
-            if (ptr_server->buffer)
-                break;
-            ptr_server = (ptr_server->next_server) ?
-                ptr_server->next_server : irc_servers;
-        }
-        if (ptr_server != SERVER(window->buffer))
-        {
-            /* save current buffer */
-            SERVER(window->buffer)->saved_buffer = window->buffer;
-            
-            /* come back to memorized chan if found */
-            if (ptr_server->saved_buffer)
-                ptr_buffer = ptr_server->saved_buffer;
-            else
-                ptr_buffer = (ptr_server->channels) ?
-                    ptr_server->channels->buffer : ptr_server->buffer;
-            if ((ptr_server->buffer == ptr_buffer)
-                && (ptr_buffer->all_servers))
-                ptr_buffer->server = ptr_server;
-            gui_window_switch_to_buffer (window, ptr_buffer);
-            gui_window_redraw_buffer (window->buffer);
+            ptr_server = SERVER(window->buffer)->next_server;
+            if (!ptr_server)
+                ptr_server = irc_servers;
+            while (ptr_server != SERVER(window->buffer))
+            {
+                if (ptr_server->buffer)
+                    break;
+                ptr_server = (ptr_server->next_server) ?
+                    ptr_server->next_server : irc_servers;
+            }
+            if (ptr_server != SERVER(window->buffer))
+            {
+                /* save current buffer */
+                SERVER(window->buffer)->saved_buffer = window->buffer;
+                
+                /* come back to memorized chan if found */
+                if (ptr_server->saved_buffer)
+                    ptr_buffer = ptr_server->saved_buffer;
+                else
+                    ptr_buffer = (ptr_server->channels) ?
+                        ptr_server->channels->buffer : ptr_server->buffer;
+                if ((ptr_server->buffer == ptr_buffer)
+                    && (ptr_buffer->all_servers))
+                    ptr_buffer->server = ptr_server;
+                gui_window_switch_to_buffer (window, ptr_buffer);
+                gui_window_redraw_buffer (window->buffer);
+            }
         }
     }
 }
@@ -1092,8 +1253,11 @@ gui_action_jump_next_server (t_gui_window *window)
  */
 
 void
-gui_action_switch_server (t_gui_window *window)
+gui_action_switch_server (t_gui_window *window, char *args)
 {
+    /* make C compiler happy */
+    (void) args;
+    
     gui_window_switch_server (window);
 }
 
@@ -1102,16 +1266,20 @@ gui_action_switch_server (t_gui_window *window)
  */
 
 void
-gui_action_scroll_previous_highlight (t_gui_window *window)
+gui_action_scroll_previous_highlight (t_gui_window *window, char *args)
 {
     t_gui_line *ptr_line;
     
-    if (window->buffer->type == BUFFER_TYPE_STANDARD)
+    /* make C compiler happy */
+    (void) args;
+    
+    if ((window->buffer->type == BUFFER_TYPE_STANDARD)
+        && (window->buffer->text_search == TEXT_SEARCH_DISABLED))
     {
         if (window->buffer->lines)
         {
             ptr_line = (window->start_line) ?
-                window->start_line->prev_line : window->buffer->last_line->prev_line;
+                window->start_line->prev_line : window->buffer->last_line;
             while (ptr_line)
             {
                 if (ptr_line->line_with_highlight)
@@ -1135,11 +1303,15 @@ gui_action_scroll_previous_highlight (t_gui_window *window)
  */
 
 void
-gui_action_scroll_next_highlight (t_gui_window *window)
+gui_action_scroll_next_highlight (t_gui_window *window, char *args)
 {
     t_gui_line *ptr_line;
     
-    if (window->buffer->type == BUFFER_TYPE_STANDARD)
+    /* make C compiler happy */
+    (void) args;
+    
+    if ((window->buffer->type == BUFFER_TYPE_STANDARD)
+        && (window->buffer->text_search == TEXT_SEARCH_DISABLED))
     {
         if (window->buffer->lines)
         {
@@ -1168,20 +1340,26 @@ gui_action_scroll_next_highlight (t_gui_window *window)
  */
 
 void
-gui_action_scroll_unread (t_gui_window *window)
+gui_action_scroll_unread (t_gui_window *window, char *args)
 {
-    if (cfg_look_read_marker &&
-        cfg_look_read_marker[0] &&
-        (window->buffer->type == BUFFER_TYPE_STANDARD) &&
-        window->buffer->last_read_line &&
-        window->buffer->last_read_line != window->buffer->last_line)
+    /* make C compiler happy */
+    (void) args;
+
+    if (window->buffer->text_search == TEXT_SEARCH_DISABLED)
     {
-        window->start_line = window->buffer->last_read_line->next_line;
-        window->start_line_pos = 0;
-        window->first_line_displayed =
-            (window->start_line == window->buffer->lines);
-        gui_chat_draw (window->buffer, 1);
-        gui_status_draw (window->buffer, 0);
+        if (cfg_look_read_marker &&
+            cfg_look_read_marker[0] &&
+            (window->buffer->type == BUFFER_TYPE_STANDARD) &&
+            window->buffer->last_read_line &&
+            window->buffer->last_read_line != window->buffer->last_line)
+        {
+            window->start_line = window->buffer->last_read_line->next_line;
+            window->start_line_pos = 0;
+            window->first_line_displayed =
+                (window->start_line == window->buffer->lines);
+            gui_chat_draw (window->buffer, 1);
+            gui_status_draw (window->buffer, 0);
+        }
     }
 }
 
@@ -1190,8 +1368,11 @@ gui_action_scroll_unread (t_gui_window *window)
  */
 
 void
-gui_action_hotlist_clear (t_gui_window *window)
+gui_action_hotlist_clear (t_gui_window *window, char *args)
 {
+    /* make C compiler happy */
+    (void) args;
+    
     if (hotlist)
     {
         hotlist_free_all ();
@@ -1205,8 +1386,11 @@ gui_action_hotlist_clear (t_gui_window *window)
  */
 
 void
-gui_action_infobar_clear (t_gui_window *window)
+gui_action_infobar_clear (t_gui_window *window, char *args)
 {
+    /* make C compiler happy */
+    (void) args;
+    
     gui_infobar_remove ();
     gui_infobar_draw (window->buffer, 1);
 }
@@ -1216,10 +1400,11 @@ gui_action_infobar_clear (t_gui_window *window)
  */
 
 void
-gui_action_refresh_screen (t_gui_window *window)
+gui_action_refresh_screen (t_gui_window *window, char *args)
 {
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) window;
+    (void) args;
     
     gui_window_refresh_screen (1);
 }
@@ -1229,8 +1414,49 @@ gui_action_refresh_screen (t_gui_window *window)
  */
 
 void
-gui_action_grab_key (t_gui_window *window)
+gui_action_grab_key (t_gui_window *window, char *args)
 {
+    /* make C compiler happy */
+    (void) args;
+    
     if (window->buffer->has_input)
         gui_keyboard_init_grab ();
 }
+
+/*
+ * gui_action_insert_string: insert a string in command line
+ */
+
+void
+gui_action_insert_string (t_gui_window *window, char *args)
+{
+    if (args)
+    {
+        gui_insert_string_input (window, args, -1);
+        gui_input_draw (window->buffer, 0);
+    }
+}
+
+/*
+ * gui_action_search_text: search text in buffer history
+ */
+
+void
+gui_action_search_text (t_gui_window *window, char *args)
+{
+    /* make C compiler happy */
+    (void) args;
+    
+    if (window->buffer->type == BUFFER_TYPE_STANDARD)
+    {
+        /* toggle search */
+        if (window->buffer->text_search == TEXT_SEARCH_DISABLED)
+            gui_buffer_search_start (window);
+        else
+        {
+            window->buffer->text_search_exact ^= 1;
+            gui_buffer_search_restart (window);
+            gui_input_draw (window->buffer, 1);
+        }
+    }
+}
