@@ -44,6 +44,10 @@
 #include "../common/util.h"
 #include "../irc/irc.h"
 
+#ifdef PLUGINS
+#include "../plugins/plugins.h"
+#endif
+
 
 t_gui_buffer *gui_buffers = NULL;           /* pointer to first buffer      */
 t_gui_buffer *last_gui_buffer = NULL;       /* pointer to last buffer       */
@@ -82,12 +86,15 @@ gui_buffer_new (t_gui_window *window, void *server, void *channel, int type,
                 int switch_to_buffer)
 {
     t_gui_buffer *new_buffer, *ptr_buffer;
+#ifdef PLUGINS
+    char buffer_str[16];
+#endif
     
 #ifdef DEBUG
     weechat_log_printf ("Creating new buffer\n");
 #endif
     
-    /* use first buffer if no server was assigned to this buffer */
+    /* use first buffer if no server is assigned to this buffer */
     if ((type == BUFFER_TYPE_STANDARD) && gui_buffers
         && (!SERVER(gui_buffers)) && (!channel))
     {
@@ -104,19 +111,20 @@ gui_buffer_new (t_gui_window *window, void *server, void *channel, int type,
         gui_buffers->completion.server = server;
         return gui_buffers;
     }
-    
+
+    /* use "all servers" buffer if found */
     if (cfg_look_one_server_buffer && (type == BUFFER_TYPE_STANDARD) &&
         gui_buffers && server && !channel)
     {
         ptr_buffer = gui_buffer_servers_search ();
         if (ptr_buffer)
         {
-            ((t_irc_server *)(server))->buffer = gui_buffers;
-            gui_buffers->server = server;
+            ((t_irc_server *)(server))->buffer = ptr_buffer;
+            ptr_buffer->server = server;
             if (switch_to_buffer)
-                gui_window_switch_to_buffer (window, gui_buffers);
-            gui_window_redraw_buffer (gui_buffers);
-            return gui_buffers;
+                gui_window_switch_to_buffer (window, ptr_buffer);
+            gui_window_redraw_buffer (ptr_buffer);
+            return ptr_buffer;
         }
     }
     
@@ -160,7 +168,7 @@ gui_buffer_new (t_gui_window *window, void *server, void *channel, int type,
         new_buffer->line_complete = 1;
         
         /* notify level */
-        new_buffer->notify_level = channel_get_notify_level (server, channel);
+        new_buffer->notify_level = irc_channel_get_notify_level (server, channel);
         
         /* create/append to log file */
         new_buffer->log_filename = NULL;
@@ -199,6 +207,12 @@ gui_buffer_new (t_gui_window *window, void *server, void *channel, int type,
         new_buffer->ptr_history = NULL;
         new_buffer->num_history = 0;
         
+        /* text search */
+        new_buffer->text_search = TEXT_SEARCH_DISABLED;
+        new_buffer->text_search_exact = 0;
+        new_buffer->text_search_found = 0;
+        new_buffer->text_search_input = NULL;
+        
         /* add buffer to buffers queue */
         new_buffer->prev_buffer = last_gui_buffer;
         if (gui_buffers)
@@ -226,6 +240,11 @@ gui_buffer_new (t_gui_window *window, void *server, void *channel, int type,
         
         /* redraw buffer */
         gui_window_redraw_buffer (new_buffer);
+
+#ifdef PLUGINS
+        snprintf (buffer_str, sizeof (buffer_str) - 1, "%d", new_buffer->number);
+        (void) plugin_event_handler_exec ("buffer_open", buffer_str);
+#endif
     }
     else
         return NULL;
@@ -255,7 +274,7 @@ gui_buffer_search (char *server, char *channel)
     {
         if (server && server[0])
         {
-            ptr_server = server_search (server);
+            ptr_server = irc_server_search (server);
             if (!ptr_server)
                 return NULL;
         }
@@ -264,7 +283,7 @@ gui_buffer_search (char *server, char *channel)
         {
             if (ptr_server)
             {
-                ptr_channel = channel_search_any (ptr_server, channel);
+                ptr_channel = irc_channel_search_any (ptr_server, channel);
                 if (ptr_channel)
                     ptr_buffer = ptr_channel->buffer;
             }
@@ -348,6 +367,89 @@ gui_buffer_find_window (t_gui_buffer *buffer)
     return NULL;
 }
 
+/*
+ * gui_buffer_find_context: find window/buffer for a server/channel
+ */
+
+void
+gui_buffer_find_context (void *server, void *channel,
+                         t_gui_window **window, t_gui_buffer **buffer)
+{
+    t_gui_window *ptr_win;
+    
+    if (!buffer)
+        return;
+    
+    /* first find buffer */
+    *buffer = NULL;
+    if ((t_irc_channel *)channel && ((t_irc_channel *)channel)->buffer)
+        *buffer = ((t_irc_channel *)channel)->buffer;
+    else
+    {
+        if ((t_irc_server *)server && ((t_irc_server *)server)->buffer)
+            *buffer = ((t_irc_server *)server)->buffer;
+        else
+            *buffer = gui_current_window->buffer;
+    }
+    
+    /* then find first window displaying this buffer */
+    if (window)
+    {
+        *window = NULL;
+        if (gui_current_window->buffer == *buffer)
+            *window = gui_current_window;
+        else
+        {
+            for (ptr_win = gui_windows; ptr_win;
+                 ptr_win = ptr_win->next_window)
+            {
+                if (ptr_win->buffer == *buffer)
+                {
+                    *window = ptr_win;
+                    break;
+                }
+            }
+            if (!*window)
+                *window = gui_current_window;
+        }
+    }
+}
+
+/*
+ * gui_buffer_is_scrolled: return 1 if all windows displaying buffer are scrolled
+ *                         (user doesn't see end of buffer)
+ *                         return 0 if at least one window is NOT scrolled
+ */
+
+int
+gui_buffer_is_scrolled (t_gui_buffer *buffer)
+{
+    t_gui_window *ptr_win;
+    int buffer_found;
+
+    if (!buffer)
+        return 0;
+    
+    buffer_found = 0;
+    for (ptr_win = gui_windows; ptr_win; ptr_win = ptr_win->next_window)
+    {
+        if (ptr_win->buffer == buffer)
+        {
+            buffer_found = 1;
+            /* buffer found and not scrolled, exit immediately */
+            if (!ptr_win->scroll)
+                return 0;
+        }
+    }
+    
+    /* buffer found, and all windows were scrolled */
+    if (buffer_found)
+        return 1;
+    
+    /* buffer not found */
+    return 0;
+}
+
 /*
  * gui_buffer_get_dcc: get pointer to DCC buffer (DCC buffer created if not existing)
  */
@@ -378,6 +480,12 @@ gui_buffer_clear (t_gui_buffer *buffer)
 {
     t_gui_window *ptr_win;
     t_gui_line *ptr_line;
+
+    if (!buffer)
+        return;
+
+    if (buffer->type == BUFFER_TYPE_DCC)
+        return;
     
     /* remove buffer from hotlist */
     hotlist_remove_buffer (buffer);
@@ -463,9 +571,27 @@ gui_buffer_free (t_gui_buffer *buffer, int switch_to_another)
     t_gui_line *ptr_line;
     t_irc_server *ptr_server;
     int create_new;
+#ifdef PLUGINS
+    char buffer_str[16];
+#endif
+    
+#ifdef PLUGINS
+    snprintf (buffer_str, sizeof (buffer_str) - 1, "%d", buffer->number);
+    (void) plugin_event_handler_exec ("buffer_close", buffer_str);
+#endif
     
     create_new = (buffer->server || buffer->channel);
     
+    if (switch_to_another)
+    {
+        for (ptr_win = gui_windows; ptr_win; ptr_win = ptr_win->next_window)
+        {
+            if ((buffer == ptr_win->buffer) &&
+                ((buffer->next_buffer) || (buffer->prev_buffer)))
+                gui_buffer_switch_previous (ptr_win);
+        }
+    }
+    
     hotlist_remove_buffer (buffer);
     if (hotlist_initial_buffer == buffer)
         hotlist_initial_buffer = NULL;
@@ -486,16 +612,6 @@ gui_buffer_free (t_gui_buffer *buffer, int switch_to_another)
             ptr_server->saved_buffer = NULL;
     }
     
-    if (switch_to_another)
-    {
-        for (ptr_win = gui_windows; ptr_win; ptr_win = ptr_win->next_window)
-        {
-            if ((buffer == ptr_win->buffer) &&
-                ((buffer->next_buffer) || (buffer->prev_buffer)))
-                gui_buffer_switch_previous (ptr_win);
-        }
-    }
-    
     /* decrease buffer number for all next buffers */
     for (ptr_buffer = buffer->next_buffer; ptr_buffer; ptr_buffer = ptr_buffer->next_buffer)
     {
@@ -523,6 +639,9 @@ gui_buffer_free (t_gui_buffer *buffer, int switch_to_another)
     
     history_buffer_free (buffer);
     
+    if (buffer->text_search_input)
+        free (buffer->text_search_input);
+    
     /* remove buffer from buffers list */
     if (buffer->prev_buffer)
         buffer->prev_buffer->next_buffer = buffer->next_buffer;
@@ -546,7 +665,8 @@ gui_buffer_free (t_gui_buffer *buffer, int switch_to_another)
         (void) gui_buffer_new (gui_windows, NULL, NULL,
                                BUFFER_TYPE_STANDARD, 1);
     
-    gui_status_draw (gui_current_window->buffer, 1);
+    if (gui_windows && gui_current_window && gui_current_window->buffer)
+        gui_status_draw (gui_current_window->buffer, 1);
 }
 
 /*
@@ -602,6 +722,31 @@ gui_buffer_line_new (t_gui_buffer *buffer, time_t date)
     return new_line;
 }
 
+/*
+ * gui_buffer_line_search: search for text in a line
+ */
+
+int
+gui_buffer_line_search (t_gui_line *line, char *text, int case_sensitive)
+{
+    char *data;
+    int rc;
+    
+    if (!text || !text[0])
+        return 0;
+    
+    rc = 0;
+    data = (char *)gui_color_decode ((unsigned char *)line->data, 0);
+    if (data)
+    {
+        if ((case_sensitive && (strstr (data, text)))
+            || (!case_sensitive && (ascii_strcasestr (data, text))))
+            rc = 1;
+        free (data);
+    }
+    return rc;
+}
+
 /*
  * gui_buffer_merge_servers: merge server buffers in one buffer
  */
@@ -917,6 +1062,134 @@ gui_buffer_move_to_number (t_gui_buffer *buffer, int number)
     gui_window_redraw_buffer (buffer);
 }
 
+/*
+ * gui_buffer_search_text: search text in a buffer
+ */
+
+int
+gui_buffer_search_text (t_gui_window *window)
+{
+    t_gui_line *ptr_line;
+    
+    if (window->buffer->text_search == TEXT_SEARCH_BACKWARD)
+    {
+        if (window->buffer->lines
+            && window->buffer->input_buffer && window->buffer->input_buffer[0])
+        {
+            ptr_line = (window->start_line) ?
+                window->start_line->prev_line : window->buffer->last_line;
+            while (ptr_line)
+            {
+                if (gui_buffer_line_search (ptr_line, window->buffer->input_buffer,
+                                            window->buffer->text_search_exact))
+                {
+                    window->start_line = ptr_line;
+                    window->start_line_pos = 0;
+                    window->first_line_displayed =
+                        (window->start_line == window->buffer->lines);
+                    gui_chat_draw (window->buffer, 1);
+                    gui_status_draw (window->buffer, 1);
+                    return 1;
+                }
+                ptr_line = ptr_line->prev_line;
+            }
+        }
+    }
+    else if (window->buffer->text_search == TEXT_SEARCH_FORWARD)
+    {
+        if (window->buffer->lines
+            && window->buffer->input_buffer && window->buffer->input_buffer[0])
+        {
+            ptr_line = (window->start_line) ?
+                window->start_line->next_line : window->buffer->lines->next_line;
+            while (ptr_line)
+            {
+                if (gui_buffer_line_search (ptr_line, window->buffer->input_buffer,
+                                            window->buffer->text_search_exact))
+                {
+                    window->start_line = ptr_line;
+                    window->start_line_pos = 0;
+                    window->first_line_displayed =
+                        (window->start_line == window->buffer->lines);
+                    gui_chat_draw (window->buffer, 1);
+                    gui_status_draw (window->buffer, 1);
+                    return 1;
+                }
+                ptr_line = ptr_line->next_line;
+            }
+        }
+    }
+    return 0;
+}
+
+/*
+ * gui_buffer_search_start: start search in a buffer
+ */
+
+void
+gui_buffer_search_start (t_gui_window *window)
+{
+    window->buffer->text_search = TEXT_SEARCH_BACKWARD;
+    window->buffer->text_search_exact = 0;
+    window->buffer->text_search_found = 0;
+    if (window->buffer->text_search_input)
+    {
+        free (window->buffer->text_search_input);
+        window->buffer->text_search_input = NULL;
+    }
+    if (window->buffer->input_buffer && window->buffer->input_buffer[0])
+        window->buffer->text_search_input =
+            strdup (window->buffer->input_buffer);
+    gui_action_delete_line (window, NULL);
+    gui_status_draw (window->buffer, 1);
+    gui_input_draw (window->buffer, 1);
+}
+
+/*
+ * gui_buffer_search_restart: restart search (after input changes or exact
+ *                            flag (un)set)
+ */
+
+void
+gui_buffer_search_restart (t_gui_window *window)
+{
+    window->start_line = NULL;
+    window->start_line_pos = 0;
+    window->buffer->text_search = TEXT_SEARCH_BACKWARD;
+    window->buffer->text_search_found = 0;
+    if (gui_buffer_search_text (window))
+        window->buffer->text_search_found = 1;
+    else
+    {
+        gui_chat_draw (window->buffer, 1);
+        gui_status_draw (window->buffer, 1);
+    }
+}
+
+/*
+ * gui_buffer_search_stop: stop search in a buffer
+ */
+
+void
+gui_buffer_search_stop (t_gui_window *window)
+{
+    window->buffer->text_search = TEXT_SEARCH_DISABLED;
+    window->buffer->text_search = 0;
+    gui_action_delete_line (window, NULL);
+    if (window->buffer->text_search_input)
+    {
+        gui_insert_string_input (window, window->buffer->text_search_input, -1);
+        free (window->buffer->text_search_input);
+        window->buffer->text_search_input = NULL;
+    }
+    window->start_line = NULL;
+    window->start_line_pos = 0;
+    hotlist_remove_buffer (window->buffer);
+    gui_chat_draw (window->buffer, 0);
+    gui_status_draw (window->buffer, 1);
+    gui_input_draw (window->buffer, 1);
+}
+
 /*
  * gui_buffer_print_log: print buffer infos in log (usually for crash dump)
  */
@@ -954,6 +1227,10 @@ gui_buffer_print_log (t_gui_buffer *buffer)
     weechat_log_printf ("  history. . . . . . . . : 0x%X\n", buffer->history);
     weechat_log_printf ("  last_history . . . . . : 0x%X\n", buffer->last_history);
     weechat_log_printf ("  ptr_history. . . . . . : 0x%X\n", buffer->ptr_history);
+    weechat_log_printf ("  num_history. . . . . . : %d\n",   buffer->num_history);
+    weechat_log_printf ("  text_search. . . . . . : %d\n",   buffer->text_search);
+    weechat_log_printf ("  text_search_exact. . . : %d\n",   buffer->text_search_exact);
+    weechat_log_printf ("  text_search_input. . . : '%s'\n", buffer->text_search_input);
     weechat_log_printf ("  prev_buffer. . . . . . : 0x%X\n", buffer->prev_buffer);
     weechat_log_printf ("  next_buffer. . . . . . : 0x%X\n", buffer->next_buffer);
     weechat_log_printf ("\n");
