@@ -51,6 +51,10 @@
 #define NOTIFY_LEVEL_MAX        3
 #define NOTIFY_LEVEL_DEFAULT    NOTIFY_LEVEL_MAX
 
+#define TEXT_SEARCH_DISABLED    0
+#define TEXT_SEARCH_BACKWARD    1
+#define TEXT_SEARCH_FORWARD     2
+
 /* buffer structures */
 
 typedef struct t_gui_line t_gui_line;
@@ -122,6 +126,12 @@ struct t_gui_buffer
     t_history *ptr_history;         /* current command in history           */
     int num_history;                /* number of commands in history        */
     
+    /* text search */
+    int text_search;                /* text search type                     */
+    int text_search_exact;          /* exact search (case sensitive) ?      */
+    int text_search_found;          /* 1 if text found, otherwise 0         */
+    char *text_search_input;        /* input saved before text search       */
+    
     /* link to previous/next buffer */
     t_gui_buffer *prev_buffer;      /* link to previous buffer              */
     t_gui_buffer *next_buffer;      /* link to next buffer                  */
