@@ -44,6 +44,10 @@
 #include "../common/util.h"
 #include "../irc/irc.h"
 
+#ifdef PLUGINS
+#include "../plugins/plugins.h"
+#endif
+
 
 int gui_init_ok = 0;                        /* = 1 if GUI is initialized    */
 int gui_ok = 0;                             /* = 1 if GUI is ok             */
@@ -174,7 +178,8 @@ gui_add_to_line (t_gui_buffer *buffer, int type, time_t date, char *nick, char *
             gui_chat_draw_line (buffer, buffer->last_line);
             gui_chat_draw (buffer, 0);
         }
-        if (gui_add_hotlist && (buffer->num_displayed == 0))
+        if (gui_add_hotlist
+            && ((buffer->num_displayed == 0) || (gui_buffer_is_scrolled (buffer))))
         {
             if (3 - buffer->last_line->line_with_message -
                 buffer->last_line->line_with_highlight <=
@@ -603,9 +608,10 @@ gui_input_complete (t_gui_window *window)
                                                                 window->buffer->input_buffer_pos)] != ' ')
                     gui_insert_string_input (window, " ",
                                              window->buffer->input_buffer_pos);
+                else
+                    window->buffer->input_buffer_pos++;
                 if (window->buffer->completion.position >= 0)
                     window->buffer->completion.position++;
-                window->buffer->input_buffer_pos++;
             }
         }
         else
@@ -619,16 +625,18 @@ gui_input_complete (t_gui_window *window)
                              cfg_look_nick_completor, strlen (cfg_look_nick_completor)) != 0)
                     gui_insert_string_input (window, cfg_look_nick_completor,
                                              window->buffer->input_buffer_pos);
+                else
+                    window->buffer->input_buffer_pos += utf8_strlen (cfg_look_nick_completor);
                 if (window->buffer->completion.position >= 0)
                     window->buffer->completion.position += strlen (cfg_look_nick_completor);
-                window->buffer->input_buffer_pos += utf8_strlen (cfg_look_nick_completor);
                 if (window->buffer->input_buffer[utf8_real_pos (window->buffer->input_buffer,
                                                                 window->buffer->input_buffer_pos)] != ' ')
                     gui_insert_string_input (window, " ",
                                              window->buffer->input_buffer_pos);
+                else
+                    window->buffer->input_buffer_pos++;
                 if (window->buffer->completion.position >= 0)
                     window->buffer->completion.position++;
-                window->buffer->input_buffer_pos++;
             }
         }
         gui_input_draw (window->buffer, 0);
@@ -664,7 +672,7 @@ gui_exec_action_dcc (t_gui_window *window, char *actions)
                         && (DCC_IS_RECV(dcc_selected->status))
                         && (dcc_selected->status == DCC_WAITING))
                     {
-                        dcc_accept (dcc_selected);
+                        irc_dcc_accept (dcc_selected);
                     }
                     break;
                 /* cancel DCC */
@@ -673,7 +681,7 @@ gui_exec_action_dcc (t_gui_window *window, char *actions)
                     if (dcc_selected
                         && (!DCC_ENDED(dcc_selected->status)))
                     {
-                        dcc_close (dcc_selected, DCC_ABORTED);
+                        irc_dcc_close (dcc_selected, DCC_ABORTED);
                         gui_window_redraw_buffer (window->buffer);
                     }
                     break;
@@ -688,7 +696,7 @@ gui_exec_action_dcc (t_gui_window *window, char *actions)
                     {
                         ptr_dcc_next = ptr_dcc->next_dcc;
                         if (DCC_ENDED(ptr_dcc->status))
-                            dcc_free (ptr_dcc);
+                            irc_dcc_free (ptr_dcc);
                         ptr_dcc = ptr_dcc_next;
                     }
                     gui_window_redraw_buffer (window->buffer);
@@ -723,7 +731,7 @@ gui_exec_action_dcc (t_gui_window *window, char *actions)
                             window->dcc_selected = dcc_selected->next_dcc;
                         else
                             window->dcc_selected = NULL;
-                        dcc_free (dcc_selected);
+                        irc_dcc_free (dcc_selected);
                         gui_window_redraw_buffer (window->buffer);
                     }
                     break;
@@ -795,10 +803,19 @@ int
 gui_insert_string_input (t_gui_window *window, char *string, int pos)
 {
     int i, pos_start, size, length;
-    char *ptr_start;
+    char *ptr_start, *string2;
+#ifdef PLUGINS
+    char *buffer_before_insert;
+#endif
     
     if (window->buffer->has_input)
     {
+#ifdef PLUGINS
+        buffer_before_insert =
+            (window->buffer->input_buffer) ?
+            strdup (window->buffer->input_buffer) : strdup ("");
+#endif
+        
         if (pos == -1)
             pos = window->buffer->input_buffer_pos;
         
@@ -828,6 +845,23 @@ gui_insert_string_input (t_gui_window *window, char *string, int pos)
         {
             window->buffer->input_buffer_color_mask[pos_start + i] = ' ';
         }
+        
+        window->buffer->input_buffer_pos += length;
+        
+#ifdef PLUGINS
+        string2 = (char *) malloc (size + 2);
+        if (string2)
+        {
+            snprintf (string2, size + 2, "*%s", string);
+            (void) plugin_keyboard_handler_exec (string2,
+                                                 buffer_before_insert,
+                                                 window->buffer->input_buffer);
+            free (string2);
+        }
+        if (buffer_before_insert)
+            free (buffer_before_insert);
+#endif
+        
         return length;
     }
     return 0;
