@@ -37,49 +37,53 @@
 
 
 /*
- * irc_find_context: find window/buffer for a server/channel
+ * irc_display_hide_password: hide IRC password(s) in a string
  */
 
 void
-irc_find_context (t_irc_server *server, t_irc_channel *channel,
-                  t_gui_window **window, t_gui_buffer **buffer)
+irc_display_hide_password (char *string, int look_for_nickserv)
 {
-    t_gui_window *ptr_win;
-    
-    if (!buffer)
-        return;
-    
-    /* first find buffer */
-    *buffer = NULL;
-    if (channel && channel->buffer)
-        *buffer = channel->buffer;
-    else
+    char *pos_nickserv, *pos, *pos_pwd;
+
+    pos = string;
+    while (1)
     {
-        if (server && server->buffer)
-            *buffer = server->buffer;
-        else
-            *buffer = gui_current_window->buffer;
-    }
-    
-    /* then find first window displaying this buffer */
-    if (window)
-    {
-        *window = NULL;
-        if (gui_current_window->buffer == *buffer)
-            *window = gui_current_window;
+        if (look_for_nickserv)
+        {
+            pos_nickserv = strstr (pos, "nickserv ");
+            if (!pos_nickserv)
+                return;
+            pos = pos_nickserv + 9;
+            while (pos[0] == ' ')
+                pos++;
+            if ((strncmp (pos, "identify ", 9) == 0)
+                || (strncmp (pos, "register ", 9) == 0))
+                pos_pwd = pos + 9;
+            else
+                pos_pwd = NULL;
+        }
         else
         {
-            for (ptr_win = gui_windows; ptr_win;
-                 ptr_win = ptr_win->next_window)
+            pos_pwd = strstr (pos, "identify ");
+            if (!pos_pwd)
+                pos_pwd = strstr (pos, "register ");
+            if (!pos_pwd)
+                return;
+            pos_pwd += 9;
+        }
+
+        if (pos_pwd)
+        {
+            while (pos_pwd[0] == ' ')
+                pos_pwd++;
+            
+            while (pos_pwd[0] && (pos_pwd[0] != ';') && (pos_pwd[0] != ' ')
+                   && (pos_pwd[0] != '"'))
             {
-                if (ptr_win->buffer == *buffer)
-                {
-                    *window = ptr_win;
-                    break;
-                }
+                pos_pwd[0] = '*';
+                pos_pwd++;
             }
-            if (!*window)
-                *window = gui_current_window;
+            pos = pos_pwd;
         }
     }
 }
@@ -174,7 +178,7 @@ irc_display_nick (t_gui_buffer *buffer, t_irc_nick *nick, char *nickname,
         length += 2;
     if (nick && cfg_look_nickmode)
     {
-        if (nick->flags & (NICK_CHANOWNER | NICK_CHANADMIN |
+        if (nick->flags & (NICK_CHANOWNER | NICK_CHANADMIN | NICK_CHANADMIN2 |
                            NICK_OP | NICK_HALFOP | NICK_VOICE))
             length += 1;
         else if (cfg_look_nickmode_empty && !no_nickmode)
@@ -220,6 +224,9 @@ irc_display_nick (t_gui_buffer *buffer, t_irc_nick *nick, char *nickname,
         else if (nick->flags & NICK_CHANADMIN)
             gui_printf_type (buffer, type, "%s&",
                              GUI_COLOR(COLOR_WIN_NICK_OP));
+        else if (nick->flags & NICK_CHANADMIN2)
+            gui_printf_type (buffer, type, "%s!",
+                             GUI_COLOR(COLOR_WIN_NICK_OP));
         else if (nick->flags & NICK_OP)
             gui_printf_type (buffer, type, "%s@",
                              GUI_COLOR(COLOR_WIN_NICK_OP));
@@ -372,6 +379,8 @@ irc_display_mode (t_irc_server *server, t_gui_buffer *buffer,
 void
 irc_display_server (t_irc_server *server)
 {
+    char *string;
+    
     gui_printf (NULL, "\n");
     gui_printf (NULL, _("%sServer: %s%s %s[%s%s%s]\n"),
                 GUI_COLOR(COLOR_WIN_CHAT),
@@ -417,9 +426,22 @@ irc_display_server (t_irc_server *server)
                 server->realname);
     gui_printf (NULL, "  server_hostname  . . . . . : %s\n",
                 (server->hostname) ? server->hostname : "");
-    gui_printf (NULL, "  server_command . . . . . . : %s\n",
-                (server->command && server->command[0]) ?
-                server->command : "");
+    if (server->command && server->command[0])
+        string = strdup (server->command);
+    else
+        string = NULL;
+    if (string)
+    {
+        if (cfg_log_hide_nickserv_pwd)
+            irc_display_hide_password (string, 1);
+        gui_printf (NULL, "  server_command . . . . . . : %s\n",
+                    string);
+        free (string);
+    }
+    else
+        gui_printf (NULL, "  server_command . . . . . . : %s\n",
+                    (server->command && server->command[0]) ?
+                    server->command : "");
     gui_printf (NULL, "  server_command_delay . . . : %d %s\n",
                 server->command_delay,
                 _("seconds"));
