@@ -45,11 +45,11 @@ irc_mode_channel_set_nick (t_irc_channel *channel, char *nick,
     
     if (nick)
     {
-        ptr_nick = nick_search (channel, nick);
+        ptr_nick = irc_nick_search (channel, nick);
         if (ptr_nick)
         {
             NICK_SET_FLAG(ptr_nick, (set_flag == '+'), flag);
-            nick_resort (channel, ptr_nick);
+            irc_nick_resort (channel, ptr_nick);
             gui_nicklist_draw (channel->buffer, 1, 1);
         }
     }
@@ -117,15 +117,17 @@ irc_mode_channel_set (t_irc_channel *channel, char *modes)
                     break;
                 default:
                     set_flag = irc_mode_channel_get_flag (modes, pos);
-                    ptr_arg = ((argc > 0) && (current_arg >= 0)) ?
-                        argv[current_arg--] : NULL;
                     switch (pos[0])
                     {
                         case 'a': /* unrealircd specific flag */
+                            ptr_arg = ((argc > 0) && (current_arg >= 0)) ?
+                                argv[current_arg--] : NULL;
                             irc_mode_channel_set_nick (channel, ptr_arg,
                                                        set_flag, NICK_CHANADMIN);
                             break;
                         case 'h':
+                            ptr_arg = ((argc > 0) && (current_arg >= 0)) ?
+                                argv[current_arg--] : NULL;
                             irc_mode_channel_set_nick (channel, ptr_arg,
                                                        set_flag, NICK_HALFOP);
                             break;
@@ -135,24 +137,40 @@ irc_mode_channel_set (t_irc_channel *channel, char *modes)
                                 free (channel->key);
                                 channel->key = NULL;
                             }
-                            if ((set_flag == '+') && ptr_arg)
-                                channel->key = strdup (ptr_arg);
+                            if (set_flag == '+')
+                            {
+                                ptr_arg = ((argc > 0) && (current_arg >= 0)) ?
+                                    argv[current_arg--] : NULL;
+                                if (ptr_arg)
+                                    channel->key = strdup (ptr_arg);
+                            }
                             break;
                         case 'l':
                             if (set_flag == '-')
                                 channel->limit = 0;
-                            if ((set_flag == '+') && ptr_arg)
-                                channel->limit = atoi (ptr_arg);
+                            if (set_flag == '+')
+                            {
+                                ptr_arg = ((argc > 0) && (current_arg >= 0)) ?
+                                    argv[current_arg--] : NULL;
+                                if (ptr_arg)
+                                    channel->limit = atoi (ptr_arg);
+                            }
                             break;
                         case 'o':
+                            ptr_arg = ((argc > 0) && (current_arg >= 0)) ?
+                                argv[current_arg--] : NULL;
                             irc_mode_channel_set_nick (channel, ptr_arg,
                                                        set_flag, NICK_OP);
                             break;
                         case 'q': /* unrealircd specific flag */
+                            ptr_arg = ((argc > 0) && (current_arg >= 0)) ?
+                                argv[current_arg--] : NULL;
                             irc_mode_channel_set_nick (channel, ptr_arg,
                                                        set_flag, NICK_CHANOWNER);
                             break;
                         case 'v':
+                            ptr_arg = ((argc > 0) && (current_arg >= 0)) ?
+                                argv[current_arg--] : NULL;
                             irc_mode_channel_set_nick (channel, ptr_arg,
                                                        set_flag, NICK_VOICE);
                             break;
@@ -257,3 +275,20 @@ irc_mode_user_set (t_irc_server *server, char *modes)
         modes++;
     }
 }
+
+/*
+ * irc_mode_nick_prefix_allowed: return <> 0 if nick prefix is allowed by server
+ *                               for example :
+ *                                 IRC:  005 (...) PREFIX=(ov)@+
+ *                               => allowed prefixes: @+
+ */
+
+int
+irc_mode_nick_prefix_allowed (t_irc_server *server, char prefix)
+{
+    /* if server did not send any prefix info, then consider this prefix is allowed */
+    if (!server->prefix)
+        return 1;
+
+    return (strchr (server->prefix, prefix) != NULL);
+}
