@@ -192,7 +192,7 @@ void
 weechat_charset_parse_irc_msg (char *message, char **nick, char **command,
                                char **channel, char **pos_args)
 {
-    char *pos, *pos2, *pos3, *pos_tmp;
+    char *pos, *pos2, *pos3, *pos4, *pos_tmp;
     
     *nick = NULL;
     *command = NULL;
@@ -234,11 +234,9 @@ weechat_charset_parse_irc_msg (char *message, char **nick, char **command,
             pos2++;
             while (pos2[0] == ' ')
                 pos2++;
-            if (pos2[0] == ':')
-                *pos_args = pos2 + 1;
-            else
+            *pos_args = pos2;
+            if (pos2[0] != ':')
             {
-                *pos_args = pos2;
                 if ((pos2[0] == '#') || (pos2[0] == '&')
                     || (pos2[0] == '+') || (pos2[0] == '!'))
                 {
@@ -258,6 +256,21 @@ weechat_charset_parse_irc_msg (char *message, char **nick, char **command,
                         else
                             *nick = strdup (pos2);
                     }
+                    if (pos3)
+                    {
+                        pos3++;
+                        while (pos3[0] == ' ')
+                            pos3++;
+                        if ((pos3[0] == '#') || (pos3[0] == '&')
+                            || (pos3[0] == '+') || (pos3[0] == '!'))
+                        {
+                            pos4 = strchr (pos3, ' ');
+                            if (pos4)
+                                *channel = weechat_charset_strndup (pos3, pos4 - pos3);
+                            else
+                                *channel = strdup (pos3);
+                        }
+                    }
                 }
             }
         }
@@ -276,7 +289,7 @@ weechat_charset_irc_in (t_weechat_plugin *plugin, int argc, char **argv,
     char *nick, *command, *channel, *charset, *ptr_args;
     char *output;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) argc;
     (void) handler_args;
     (void) handler_pointer;
@@ -324,7 +337,7 @@ weechat_charset_irc_out (t_weechat_plugin *plugin, int argc, char **argv,
     char *nick, *command, *channel, *charset, *ptr_args;
     char *output;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) argc;
     (void) handler_args;
     (void) handler_pointer;
@@ -457,7 +470,7 @@ weechat_charset_cmd (t_weechat_plugin *plugin,
     if (cmd_argc < 3)
         return PLUGIN_RC_KO;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) handler_args;
     (void) handler_pointer;
     
@@ -609,7 +622,7 @@ weechat_plugin_init (t_weechat_plugin *plugin)
 void
 weechat_plugin_end (t_weechat_plugin *plugin)
 {
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) plugin;
     
     if (weechat_charset_terminal)
