@@ -52,7 +52,7 @@ int
 weechat_ascii_strcasecmp (t_weechat_plugin *plugin,
                           char *string1, char *string2)
 {
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) plugin;
     
     return ascii_strcasecmp (string1, string2);
@@ -67,7 +67,7 @@ int
 weechat_ascii_strncasecmp (t_weechat_plugin *plugin,
                           char *string1, char *string2, int max)
 {
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) plugin;
     
     return ascii_strncasecmp (string1, string2, max);
@@ -82,7 +82,7 @@ weechat_explode_string (t_weechat_plugin *plugin, char *string,
                         char *separators, int num_items_max,
                         int *num_items)
 {
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) plugin;
     
     if (!plugin || !string || !separators || !num_items)
@@ -98,7 +98,7 @@ weechat_explode_string (t_weechat_plugin *plugin, char *string,
 void
 weechat_free_exploded_string (t_weechat_plugin *plugin, char **exploded_string)
 {
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) plugin;
     
     free_exploded_string (exploded_string);
@@ -114,7 +114,7 @@ weechat_plugin_mkdir_home (t_weechat_plugin *plugin, char *directory)
     char *dir_name;
     int dir_length;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) plugin;
     
     if (!directory)
@@ -166,7 +166,7 @@ weechat_plugin_print (t_weechat_plugin *plugin,
     t_gui_buffer *ptr_buffer;
     va_list argptr;
     static char buf[8192];
-    char *buf2;
+    char *buf2, *buf3;
     
     if (!plugin || !message)
         return;
@@ -177,10 +177,14 @@ weechat_plugin_print (t_weechat_plugin *plugin,
     va_end (argptr);
 
     buf2 = weechat_iconv_to_internal (plugin->charset, buf);
+    buf3 = (char *)gui_color_decode ((unsigned char *)((buf2) ? buf2 : buf),
+                                     cfg_irc_colors_receive);
     irc_display_prefix (NULL, ptr_buffer, PREFIX_PLUGIN);
-    gui_printf (ptr_buffer, "%s\n", (buf2) ? buf2 : buf);
+    gui_printf (ptr_buffer, "%s\n", (buf3) ? buf3 : ((buf2) ? buf2 : buf));
     if (buf2)
         free (buf2);
+    if (buf3)
+        free (buf3);
 }
 
 /*
@@ -192,7 +196,7 @@ weechat_plugin_print_server (t_weechat_plugin *plugin, char *message, ...)
 {
     va_list argptr;
     static char buf[8192];
-    char *buf2;
+    char *buf2, *buf3;
     
     if (!plugin || !message)
         return;
@@ -202,10 +206,14 @@ weechat_plugin_print_server (t_weechat_plugin *plugin, char *message, ...)
     va_end (argptr);
 
     buf2 = weechat_iconv_to_internal (plugin->charset, buf);
+    buf3 = (char *)gui_color_decode ((unsigned char *)((buf2) ? buf2 : buf),
+                                     cfg_irc_colors_receive);
     irc_display_prefix (NULL, NULL, PREFIX_PLUGIN);
-    gui_printf (NULL, "%s\n", (buf2) ? buf2 : buf);
+    gui_printf (NULL, "%s\n", (buf3) ? buf3 : ((buf2) ? buf2 : buf));
     if (buf2)
         free (buf2);
+    if (buf3)
+        free (buf3);
 }
 
 /*
@@ -356,6 +364,22 @@ weechat_plugin_keyboard_handler_add (t_weechat_plugin *plugin,
     return NULL;
 }
 
+/*
+ * weechat_plugin_event_handler_add: add an event handler
+ */
+
+t_plugin_handler *
+weechat_plugin_event_handler_add (t_weechat_plugin *plugin, char *event,
+                                  t_plugin_handler_func *handler_func,
+                                  char *handler_args, void *handler_pointer)
+{
+    if (plugin && event && handler_func)
+        return plugin_event_handler_add (plugin, event, handler_func,
+                                         handler_args, handler_pointer);
+    
+    return NULL;
+}
+
 /*
  * weechat_plugin_handler_remove: remove a WeeChat handler
  */
@@ -571,6 +595,15 @@ weechat_plugin_get_info (t_weechat_plugin *plugin, char *info, char *server)
         if (ptr_server && ptr_server->is_connected && ptr_server->name)
             return strdup (ptr_server->name);
     }
+    else if (ascii_strcasecmp (info, "type") == 0)
+    {
+        return_str = (char *) malloc (32);
+        if (!return_str)
+            return NULL;
+        snprintf (return_str, 32, "%d",
+                  gui_current_window->buffer->type);
+        return return_str;
+    }
     else if (ascii_strcasecmp (info, "away") == 0)
     {
         if (ptr_server && ptr_server->is_connected && ptr_server->is_away)
@@ -728,7 +761,7 @@ weechat_plugin_get_config (t_weechat_plugin *plugin, char *option)
     char option_name[256];
     void *ptr_option_value;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) plugin;
     
     for (i = 0; i < CONFIG_NUMBER_SECTIONS; i++)
@@ -785,7 +818,7 @@ weechat_plugin_set_config (t_weechat_plugin *plugin, char *option, char *value)
     t_irc_server *ptr_server;
     t_config_option *ptr_option;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) plugin;
     
     if (!option || !value)
@@ -800,7 +833,7 @@ weechat_plugin_set_config (t_weechat_plugin *plugin, char *option, char *value)
         {
             strncpy (server_name, option, pos - option);
             server_name[pos - option] = '\0';
-            ptr_server = server_search (server_name);
+            ptr_server = irc_server_search (server_name);
             free (server_name);
             if (ptr_server)
                 return (config_set_server_value (ptr_server, pos + 1, value) == 0);
@@ -866,8 +899,8 @@ weechat_plugin_set_plugin_config (t_weechat_plugin *plugin, char *option, char *
 
 /*
  * weechat_plugin_get_server_info: get list of server info
- *                          WARNING: caller has to free structure returned
- *                          by this function after use
+ *                                 WARNING: caller has to free structure returned
+ *                                 by this function after use
  */
 
 t_plugin_server_info *
@@ -970,6 +1003,8 @@ weechat_plugin_free_server_info (t_weechat_plugin *plugin, t_plugin_server_info
             free (server_info->notify_levels);
 	if (server_info->nick)
             free (server_info->nick);
+	if (server_info->nick_modes)
+            free (server_info->nick_modes);
         new_server_info = server_info->next_server;
         free (server_info);
         server_info = new_server_info;
@@ -992,7 +1027,7 @@ weechat_plugin_get_channel_info (t_weechat_plugin *plugin, char *server)
     if (!plugin || !server || !server[0])
 	return NULL;
     
-    ptr_server = server_search (server);
+    ptr_server = irc_server_search (server);
     if (!ptr_server)
 	return NULL;
 
@@ -1076,11 +1111,11 @@ weechat_plugin_get_nick_info (t_weechat_plugin *plugin, char *server, char *chan
     if (!plugin || !server || !server[0] || !channel || !channel[0])
 	return NULL;
     
-    ptr_server = server_search (server);
+    ptr_server = irc_server_search (server);
     if (!ptr_server)
 	return NULL;
 
-    ptr_channel = channel_search (ptr_server, channel);
+    ptr_channel = irc_channel_search (ptr_server, channel);
     if (!ptr_channel)
 	return NULL;
     
