@@ -151,7 +151,7 @@ weechat_lua_timer_handler (t_weechat_plugin *plugin,
                            int argc, char **argv,
                            char *handler_args, void *handler_pointer)
 {
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) argc;
     (void) argv;
     int *r;
@@ -200,6 +200,36 @@ weechat_lua_keyboard_handler (t_weechat_plugin *plugin,
         return PLUGIN_RC_KO;
 }
 
+/*
+ * weechat_lua_event_handler: general event handler for Lua
+ */
+
+int
+weechat_lua_event_handler (t_weechat_plugin *plugin,
+                           int argc, char **argv,
+                           char *handler_args, void *handler_pointer)
+{
+    int *r;
+    int ret;
+
+    if (argc >= 1)
+    {
+        r = (int *) weechat_lua_exec (plugin, (t_plugin_script *)handler_pointer,
+                                      SCRIPT_EXEC_INT,
+                                      handler_args, argv[0], NULL, NULL);
+        if (r == NULL)
+            ret = PLUGIN_RC_KO;
+        else
+        {
+            ret = *r;
+            free (r);
+        }
+        return ret;
+    }
+    else
+        return PLUGIN_RC_KO;
+}
+
 /*
  * weechat_lua_modifier: general modifier for Lua
  */
@@ -227,7 +257,7 @@ weechat_lua_register (lua_State *L)
     const char *name, *version, *shutdown_func, *description, *charset;
     int n;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) L;
 
     lua_current_script = NULL;
@@ -310,12 +340,12 @@ weechat_lua_register (lua_State *L)
  */
 
 static int
-weechat_lua_set_charset  (lua_State *L)
+weechat_lua_set_charset (lua_State *L)
 {
     const char *charset;
     int n;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) L;
      
     if (!lua_current_script)
@@ -355,12 +385,12 @@ weechat_lua_set_charset  (lua_State *L)
  */
 
 static int
-weechat_lua_print  (lua_State *L)
+weechat_lua_print (lua_State *L)
 {
     const char *message, *channel_name, *server_name;
     int n;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) L;
     
     if (!lua_current_script)
@@ -414,12 +444,12 @@ weechat_lua_print  (lua_State *L)
  */
 
 static int
-weechat_lua_print_server  (lua_State *L)
+weechat_lua_print_server (lua_State *L)
 {
     const char *message;
     int n;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) L;
     
     if (!lua_current_script)
@@ -457,12 +487,12 @@ weechat_lua_print_server  (lua_State *L)
  */
 
 static int
-weechat_lua_print_infobar  (lua_State *L)
+weechat_lua_print_infobar (lua_State *L)
 {
     const char *message;
     int delay, n;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) L;
     
     if (!lua_current_script)
@@ -502,11 +532,11 @@ weechat_lua_print_infobar  (lua_State *L)
  */
 
 static int
-weechat_lua_remove_infobar  (lua_State *L)
+weechat_lua_remove_infobar (lua_State *L)
 {
     int n, how_many;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) L;
     
     if (!lua_current_script)
@@ -536,12 +566,12 @@ weechat_lua_remove_infobar  (lua_State *L)
  */
 
 static int
-weechat_lua_log  (lua_State *L)
+weechat_lua_log (lua_State *L)
 {
     const char *message, *channel_name, *server_name;
     int n;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) L;
     
     if (!lua_current_script)
@@ -595,12 +625,12 @@ weechat_lua_log  (lua_State *L)
  */
 
 static int
-weechat_lua_command  (lua_State *L)
+weechat_lua_command (lua_State *L)
 {
     const char *command, *channel_name, *server_name;
     int n;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) L;
      
     if (!lua_current_script)
@@ -654,12 +684,12 @@ weechat_lua_command  (lua_State *L)
  */
 
 static int
-weechat_lua_add_message_handler  (lua_State *L)
+weechat_lua_add_message_handler (lua_State *L)
 {
     const char *irc_command, *function;
     int n;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) L;
     
     if (!lua_current_script)
@@ -706,13 +736,13 @@ weechat_lua_add_message_handler  (lua_State *L)
  */
 
 static int
-weechat_lua_add_command_handler  (lua_State *L)
+weechat_lua_add_command_handler (lua_State *L)
 {
     const char *command, *function, *description, *arguments, *arguments_description;
     const char *completion_template;
     int n;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) L;
         
     if (!lua_current_script)
@@ -778,13 +808,13 @@ weechat_lua_add_command_handler  (lua_State *L)
  */
 
 static int
-weechat_lua_add_timer_handler  (lua_State *L)
+weechat_lua_add_timer_handler (lua_State *L)
 {
     int interval;
     const char *function;
     int n;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) L;
     
     if (!lua_current_script)
@@ -831,12 +861,12 @@ weechat_lua_add_timer_handler  (lua_State *L)
  */
 
 static int
-weechat_lua_add_keyboard_handler  (lua_State *L)
+weechat_lua_add_keyboard_handler (lua_State *L)
 {
     const char *function;
     int n;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) L;
     
     if (!lua_current_script)
@@ -876,6 +906,58 @@ weechat_lua_add_keyboard_handler  (lua_State *L)
     return 1;
 }
 
+/*
+ * weechat_lua_add_event_handler: add handler for events
+ */
+
+static int
+weechat_lua_add_event_handler (lua_State *L)
+{
+    const char *event, *function;
+    int n;
+    
+    /* make C compiler happy */
+    (void) L;
+    
+    if (!lua_current_script)
+    {
+        lua_plugin->print_server (lua_plugin,
+                                  "Lua error: unable to add event handler, "
+                                  "script not initialized");
+	lua_pushnumber (lua_current_interpreter, 0);
+	return 1;
+    }
+    
+    event = NULL;
+    function = NULL;
+    
+    n = lua_gettop (lua_current_interpreter);
+
+    if (n != 2)
+    {
+	lua_plugin->print_server (lua_plugin,
+                                  "Lua error: wrong parameters for "
+                                  "\"add_event_handler\" function");
+        lua_pushnumber (lua_current_interpreter, 0);
+	return 1;
+    }
+
+    event = lua_tostring (lua_current_interpreter, -2);
+    function = lua_tostring (lua_current_interpreter, -1);
+    
+    if (!lua_plugin->event_handler_add (lua_plugin, (char *) event,
+                                        weechat_lua_event_handler,
+                                        (char *) function,
+                                        (void *)lua_current_script))
+    {
+	lua_pushnumber (lua_current_interpreter, 0);
+	return 1;
+    }
+
+    lua_pushnumber (lua_current_interpreter, 1);
+    return 1;
+}
+
 /*
  * weechat_lua_remove_handler: remove a command/message handler
  */
@@ -886,7 +968,7 @@ weechat_lua_remove_handler (lua_State *L)
     const char *command, *function;
     int n;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) L;
      
     if (!lua_current_script)
@@ -932,7 +1014,7 @@ weechat_lua_remove_timer_handler (lua_State *L)
     const char *function;
     int n;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) L;
      
     if (!lua_current_script)
@@ -976,7 +1058,7 @@ weechat_lua_remove_keyboard_handler (lua_State *L)
     const char *function;
     int n;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) L;
      
     if (!lua_current_script)
@@ -1010,6 +1092,50 @@ weechat_lua_remove_keyboard_handler (lua_State *L)
     return 1;
 }
 
+/*
+ * weechat_lua_remove_event_handler: remove an event handler
+ */
+
+static int
+weechat_lua_remove_event_handler (lua_State *L)
+{
+    const char *function;
+    int n;
+    
+    /* make C compiler happy */
+    (void) L;
+     
+    if (!lua_current_script)
+    {
+        lua_plugin->print_server (lua_plugin,
+                                  "Lua error: unable to remove event handler, "
+                                  "script not initialized");
+        lua_pushnumber (lua_current_interpreter, 0);
+	return 1;
+    }
+    
+    function = NULL;
+ 
+    n = lua_gettop (lua_current_interpreter);
+    
+    if (n != 1)
+    {
+	lua_plugin->print_server (lua_plugin,
+                                  "Lua error: wrong parameters for "
+                                  "\"remove_event_handler\" function");
+        lua_pushnumber (lua_current_interpreter, 0);
+	return 1;
+    }
+
+    function = lua_tostring (lua_current_interpreter, -1);
+    
+    weechat_script_remove_event_handler (lua_plugin, lua_current_script,
+                                         (char *) function);
+    
+    lua_pushnumber (lua_current_interpreter, 1);
+    return 1;
+}
+
 /*
  * weechat_lua_add_modifier: add a modifier
  */
@@ -1020,7 +1146,7 @@ weechat_lua_add_modifier (lua_State *L)
     const char *type, *command, *function;
     int n;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) L;
     
     if (!lua_current_script)
@@ -1074,7 +1200,7 @@ weechat_lua_remove_modifier (lua_State *L)
     const char *type, *command, *function;
     int n;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) L;
      
     if (!lua_current_script)
@@ -1118,13 +1244,13 @@ weechat_lua_remove_modifier (lua_State *L)
  */
 
 static int
-weechat_lua_get_info  (lua_State *L)
+weechat_lua_get_info (lua_State *L)
 {
     const char *arg, *server_name;
     char *info;
     int n;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) L;
     
     if (!lua_current_script)
@@ -1172,7 +1298,7 @@ weechat_lua_get_info  (lua_State *L)
  */
 
 static int
-weechat_lua_get_dcc_info  (lua_State *L)
+weechat_lua_get_dcc_info (lua_State *L)
 {
     t_plugin_dcc_info *dcc_info, *ptr_dcc;
     char timebuffer1[64];
@@ -1180,7 +1306,7 @@ weechat_lua_get_dcc_info  (lua_State *L)
     struct in_addr in;
     int i;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) L;
     
     if (!lua_current_script)
@@ -1289,13 +1415,13 @@ weechat_lua_get_dcc_info  (lua_State *L)
  */
 
 static int
-weechat_lua_get_config  (lua_State *L)
+weechat_lua_get_config (lua_State *L)
 {
     const char *option;
     char *return_value;
     int n;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) L;
      
     if (!lua_current_script)
@@ -1336,12 +1462,12 @@ weechat_lua_get_config  (lua_State *L)
  */
 
 static int
-weechat_lua_set_config  (lua_State *L)
+weechat_lua_set_config (lua_State *L)
 {
     const char *option, *value;
     int n;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) L;
     
     if (!lua_current_script)
@@ -1383,13 +1509,13 @@ weechat_lua_set_config  (lua_State *L)
  */
 
 static int
-weechat_lua_get_plugin_config  (lua_State *L)
+weechat_lua_get_plugin_config (lua_State *L)
 {
     const char *option;
     char *return_value;
     int n;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) L;
      
     if (!lua_current_script)
@@ -1432,12 +1558,12 @@ weechat_lua_get_plugin_config  (lua_State *L)
  */
 
 static int
-weechat_lua_set_plugin_config  (lua_State *L)
+weechat_lua_set_plugin_config (lua_State *L)
 {
     const char *option, *value;
     int n;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) L;
  	
     if (!lua_current_script)
@@ -1481,12 +1607,12 @@ weechat_lua_set_plugin_config  (lua_State *L)
  */
 
 static int
-weechat_lua_get_server_info  (lua_State *L)
+weechat_lua_get_server_info (lua_State *L)
 {
     t_plugin_server_info *server_info, *ptr_server;
     char timebuffer[64];
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) L;
     
     if (!lua_current_script)
@@ -1605,7 +1731,7 @@ weechat_lua_get_server_info  (lua_State *L)
         lua_pushstring (lua_current_interpreter, "nick_modes");
 	lua_pushstring (lua_current_interpreter, ptr_server->nick_modes);
 	lua_rawset (lua_current_interpreter, -3);
-	
+        
 	lua_pushstring (lua_current_interpreter, "away_time");
 	lua_pushstring (lua_current_interpreter, timebuffer);
 	lua_rawset (lua_current_interpreter, -3);
@@ -1627,13 +1753,13 @@ weechat_lua_get_server_info  (lua_State *L)
  */
 
 static int
-weechat_lua_get_channel_info  (lua_State *L)
+weechat_lua_get_channel_info (lua_State *L)
 {
     t_plugin_channel_info *channel_info, *ptr_channel;
     const char *server;
     int n;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) L;
  
     if (!lua_current_script)
@@ -1711,13 +1837,13 @@ weechat_lua_get_channel_info  (lua_State *L)
  */
 
 static int
-weechat_lua_get_nick_info  (lua_State *L)
+weechat_lua_get_nick_info (lua_State *L)
 {
     t_plugin_nick_info *nick_info, *ptr_nick;
     const char *server, *channel;
     int n;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) L;
      
     if (!lua_current_script)
@@ -1788,7 +1914,7 @@ weechat_lua_get_irc_color (lua_State *L)
     const char *color;
     int n;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) L;
      
     if (!lua_current_script)
@@ -1825,12 +1951,12 @@ weechat_lua_get_irc_color (lua_State *L)
  */
 
 static int
-weechat_lua_get_window_info  (lua_State *L)
+weechat_lua_get_window_info (lua_State *L)
 {
     t_plugin_window_info *window_info, *ptr_window;
     int i;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) L;
     
     if (!lua_current_script)
@@ -1897,11 +2023,11 @@ weechat_lua_get_window_info  (lua_State *L)
  */
 
 static int
-weechat_lua_get_buffer_info  (lua_State *L)
+weechat_lua_get_buffer_info (lua_State *L)
 {
     t_plugin_buffer_info *buffer_info, *ptr_buffer;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) L;
     
     if (!lua_current_script)
@@ -1929,7 +2055,7 @@ weechat_lua_get_buffer_info  (lua_State *L)
 	lua_pushstring (lua_current_interpreter, "type");
 	lua_pushnumber (lua_current_interpreter, ptr_buffer->type);
 	lua_rawset (lua_current_interpreter, -3);
-	
+        
 	lua_pushstring (lua_current_interpreter, "num_displayed");
 	lua_pushnumber (lua_current_interpreter, ptr_buffer->num_displayed);
 	lua_rawset (lua_current_interpreter, -3);
@@ -1966,14 +2092,14 @@ weechat_lua_get_buffer_info  (lua_State *L)
  */
 
 static int
-weechat_lua_get_buffer_data  (lua_State *L)
+weechat_lua_get_buffer_data (lua_State *L)
 {
     t_plugin_buffer_line *buffer_data, *ptr_data;
     const char *server, *channel;
     char timebuffer[64];
     int i, n;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) L;
     
     if (!lua_current_script)
@@ -1989,26 +2115,18 @@ weechat_lua_get_buffer_data  (lua_State *L)
     channel = NULL;
     
     n = lua_gettop (lua_current_interpreter);
-
-    switch (n)
+    if (n != 2)
     {
-    case 0:
-	break;
-    case 1:
-	server = lua_tostring (lua_current_interpreter, -1);
-	break;
-    case 2:
-	server  = lua_tostring (lua_current_interpreter, -2);
-	channel = lua_tostring (lua_current_interpreter, -1);
-	break;
-    default:
 	lua_plugin->print_server (lua_plugin,
 				  "Lua error: wrong parameters for "
 				  "\"get_buffer_data\" function");
 	lua_pushnumber (lua_current_interpreter, 0);
 	return 1;
     }
-
+    
+    server  = lua_tostring (lua_current_interpreter, -2);
+    channel = lua_tostring (lua_current_interpreter, -1);
+    
     buffer_data = lua_plugin->get_buffer_data (lua_plugin, (char *) server, (char *) channel);
     if (!buffer_data)
     {
@@ -2053,9 +2171,9 @@ weechat_lua_get_buffer_data  (lua_State *L)
  */
 
 static int
-weechat_lua_constant_plugin_rc_ok  (lua_State *L)
+weechat_lua_constant_plugin_rc_ok (lua_State *L)
 {
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) L;
     
     lua_pushnumber (lua_current_interpreter, PLUGIN_RC_OK);
@@ -2063,9 +2181,9 @@ weechat_lua_constant_plugin_rc_ok  (lua_State *L)
 }
 
 static int
-weechat_lua_constant_plugin_rc_ko  (lua_State *L)
+weechat_lua_constant_plugin_rc_ko (lua_State *L)
 {
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) L;
     
     lua_pushnumber (lua_current_interpreter, PLUGIN_RC_KO);
@@ -2073,9 +2191,9 @@ weechat_lua_constant_plugin_rc_ko  (lua_State *L)
 }
     
 static int
-weechat_lua_constant_plugin_rc_ok_ignore_weechat  (lua_State *L)
+weechat_lua_constant_plugin_rc_ok_ignore_weechat (lua_State *L)
 {
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) L;
     
     lua_pushnumber (lua_current_interpreter, PLUGIN_RC_OK_IGNORE_WEECHAT);
@@ -2083,9 +2201,9 @@ weechat_lua_constant_plugin_rc_ok_ignore_weechat  (lua_State *L)
 }
 
 static int
-weechat_lua_constant_plugin_rc_ok_ignore_plugins  (lua_State *L)
+weechat_lua_constant_plugin_rc_ok_ignore_plugins (lua_State *L)
 {
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) L;
     
     lua_pushnumber (lua_current_interpreter, PLUGIN_RC_OK_IGNORE_PLUGINS);
@@ -2093,15 +2211,25 @@ weechat_lua_constant_plugin_rc_ok_ignore_plugins  (lua_State *L)
 }
 
 static int
-weechat_lua_constant_plugin_rc_ok_ignore_all  (lua_State *L)
+weechat_lua_constant_plugin_rc_ok_ignore_all (lua_State *L)
 {
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) L;
     
     lua_pushnumber (lua_current_interpreter, PLUGIN_RC_OK_IGNORE_ALL);
     return 1;
 }
 
+static int
+weechat_lua_constant_plugin_rc_ok_with_highlight (lua_State *L)
+{
+    /* make C compiler happy */
+    (void) L;
+    
+    lua_pushnumber (lua_current_interpreter, PLUGIN_RC_OK_WITH_HIGHLIGHT);
+    return 1;
+}
+
 /*
  * Lua subroutines
  */
@@ -2120,9 +2248,11 @@ const struct luaL_reg weechat_lua_funcs[] = {
     { "add_command_handler", weechat_lua_add_command_handler },
     { "add_timer_handler", weechat_lua_add_timer_handler },
     { "add_keyboard_handler", weechat_lua_add_keyboard_handler },
+    { "add_event_handler", weechat_lua_add_event_handler },
     { "remove_handler", weechat_lua_remove_handler },
     { "remove_timer_handler", weechat_lua_remove_timer_handler },
     { "remove_keyboard_handler", weechat_lua_remove_keyboard_handler },
+    { "remove_event_handler", weechat_lua_remove_event_handler },
     { "add_modifier", weechat_lua_add_modifier },
     { "remove_modifier", weechat_lua_remove_modifier },
     { "get_info", weechat_lua_get_info },
@@ -2144,6 +2274,7 @@ const struct luaL_reg weechat_lua_funcs[] = {
     { "PLUGIN_RC_OK_IGNORE_WEECHAT", weechat_lua_constant_plugin_rc_ok_ignore_weechat },
     { "PLUGIN_RC_OK_IGNORE_PLUGINS", weechat_lua_constant_plugin_rc_ok_ignore_plugins },
     { "PLUGIN_RC_OK_IGNORE_ALL", weechat_lua_constant_plugin_rc_ok_ignore_all },
+    { "PLUGIN_RC_OK_WITH_HIGHLIGHT", weechat_lua_constant_plugin_rc_ok_with_highlight },
     { NULL, NULL }
 };
 
@@ -2203,7 +2334,7 @@ weechat_lua_load (t_weechat_plugin *plugin, char *filename)
         plugin->print_server (plugin,
                               "Lua warning: unable to redirect stdout and stderr");
     
-    lua_current_script_filename = strdup (filename);
+    lua_current_script_filename = filename;
     
     if (luaL_loadfile (lua_current_interpreter, filename) != 0)
     {
@@ -2213,7 +2344,6 @@ weechat_lua_load (t_weechat_plugin *plugin, char *filename)
 	plugin->print_server (plugin,
                               "Lua error: %s",
                               lua_tostring (lua_current_interpreter, -1));
-        free (lua_current_script_filename);
         lua_close (lua_current_interpreter);
         fclose (fp);
         return 0;
@@ -2227,7 +2357,6 @@ weechat_lua_load (t_weechat_plugin *plugin, char *filename)
 	plugin->print_server (plugin,
                               "Lua error: %s",
                               lua_tostring (lua_current_interpreter, -1));
-        free (lua_current_script_filename);
         lua_close (lua_current_interpreter);
         fclose (fp);
 	/* if script was registered, removing from list */
@@ -2235,9 +2364,7 @@ weechat_lua_load (t_weechat_plugin *plugin, char *filename)
 	    weechat_script_remove (plugin, &lua_scripts, lua_current_script);
         return 0;
     }
-
     fclose (fp);
-    free (lua_current_script_filename);
     
     if (lua_current_script == NULL)
     {
@@ -2336,7 +2463,7 @@ weechat_lua_cmd (t_weechat_plugin *plugin,
     t_plugin_handler *ptr_handler;
     t_plugin_modifier *ptr_modifier;
 
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) handler_args;
     (void) handler_pointer;
     
@@ -2447,6 +2574,25 @@ weechat_lua_cmd (t_weechat_plugin *plugin,
             if (!handler_found)
                 plugin->print_server (plugin, "  (none)");
             
+            /* list Lua event handlers */
+            plugin->print_server (plugin, "");
+            plugin->print_server (plugin, "Lua event handlers:");
+            handler_found = 0;
+            for (ptr_handler = plugin->handlers;
+                 ptr_handler; ptr_handler = ptr_handler->next_handler)
+            {
+                if ((ptr_handler->type == PLUGIN_HANDLER_EVENT)
+                    && (ptr_handler->handler_args))
+                {
+                    handler_found = 1;
+                    plugin->print_server (plugin, "  %s => Lua(%s)",
+                                          ptr_handler->event,
+                                          ptr_handler->handler_args);
+                }
+            }
+            if (!handler_found)
+                plugin->print_server (plugin, "  (none)");
+            
 	    /* list Lua modifiers */
 	    plugin->print_server (plugin, "");
             plugin->print_server (plugin, "Lua modifiers:");
