@@ -145,7 +145,9 @@ weechat_perl_exec (t_weechat_plugin *plugin,
     SAVETMPS;
     PUSHMARK(sp);
     
-    perl_current_script = script;
+    /* are we loading the script file ? */
+    if (strcmp (function, "weechat_perl_load_eval_file") != 0)
+	perl_current_script = script;
 
     if (arg1)
     {
@@ -176,7 +178,7 @@ weechat_perl_exec (t_weechat_plugin *plugin,
     if (SvTRUE (ERRSV))
     {
         plugin->print_server (plugin, "Perl error: %s", SvPV_nolen (ERRSV));
-        POPs; /* poping the 'undef' */	
+        (void) POPs; /* poping the 'undef' */	
 	mem_err = 0;
     }
     else
@@ -224,7 +226,7 @@ weechat_perl_exec (t_weechat_plugin *plugin,
     if (ret_value == NULL && mem_err == 1)
     {
 	plugin->print_server (plugin,
-                              "Python error: unable to alloc memory in function \"%s\"",
+                              "Perl error: unable to alloc memory in function \"%s\"",
                               function);
 	return NULL;
     }
@@ -273,7 +275,7 @@ weechat_perl_timer_handler (t_weechat_plugin *plugin,
 {
     int *r, ret;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) argc;
     (void) argv;
     
@@ -320,6 +322,35 @@ weechat_perl_keyboard_handler (t_weechat_plugin *plugin,
         return PLUGIN_RC_KO;
 }
 
+/*
+ * weechat_perl_event_handler: general event handler for Perl
+ */
+
+int
+weechat_perl_event_handler (t_weechat_plugin *plugin,
+                            int argc, char **argv,
+                            char *handler_args, void *handler_pointer)
+{
+    int *r, ret;
+    
+    if (argc >= 1)
+    {
+        r = (int *) weechat_perl_exec (plugin, (t_plugin_script *)handler_pointer,
+                                       SCRIPT_EXEC_INT,
+                                       handler_args, argv[0], NULL, NULL);
+        if (r == NULL)
+            ret = PLUGIN_RC_KO;
+        else
+        {
+            ret = *r;
+            free (r);
+        }
+        return ret;
+    }
+    else
+        return PLUGIN_RC_KO;
+}
+
 /*
  * weechat_perl_modifier: general modifier for Perl
  */
@@ -346,7 +377,7 @@ static XS (XS_weechat_register)
     char *name, *version, *shutdown_func, *description, *charset;
     dXSARGS;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) items;
     (void) cv;
 
@@ -407,7 +438,7 @@ static XS (XS_weechat_set_charset)
 {
     dXSARGS;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) cv;
     
     if (!perl_current_script)
@@ -442,7 +473,7 @@ static XS (XS_weechat_print)
     char *message, *channel_name, *server_name;
     dXSARGS;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) cv;
 
     if (!perl_current_script)
@@ -489,7 +520,7 @@ static XS (XS_weechat_print_server)
     char *message;
     dXSARGS;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) cv;
 
     if (!perl_current_script)
@@ -524,7 +555,7 @@ static XS (XS_weechat_print_infobar)
 {
     dXSARGS;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) cv;
     
     if (!perl_current_script)
@@ -558,7 +589,7 @@ static XS (XS_weechat_remove_infobar)
 {
     dXSARGS;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) cv;
     
     if (!perl_current_script)
@@ -584,7 +615,7 @@ static XS (XS_weechat_log)
     char *message, *channel_name, *server_name;
     dXSARGS;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) cv;
 
     if (!perl_current_script)
@@ -631,7 +662,7 @@ static XS (XS_weechat_command)
     char *channel_name, *server_name;
     dXSARGS;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) cv;
     
     if (!perl_current_script)
@@ -676,7 +707,7 @@ static XS (XS_weechat_add_message_handler)
     char *irc_command, *function;
     dXSARGS;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) cv;
     
     if (!perl_current_script)
@@ -716,7 +747,7 @@ static XS (XS_weechat_add_command_handler)
     char *completion_template;
     dXSARGS;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) cv;
     
     if (!perl_current_script)
@@ -766,7 +797,7 @@ static XS (XS_weechat_add_timer_handler)
     char *function;
     dXSARGS;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) cv;
     
     if (!perl_current_script)
@@ -805,7 +836,7 @@ static XS (XS_weechat_add_keyboard_handler)
     char *function;
     dXSARGS;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) cv;
     
     if (!perl_current_script)
@@ -835,6 +866,45 @@ static XS (XS_weechat_add_keyboard_handler)
     XSRETURN_NO;
 }
 
+/*
+ * weechat::add_event_handler: add an event handler
+ */
+
+static XS (XS_weechat_add_event_handler)
+{
+    char *event, *function;
+    dXSARGS;
+    
+    /* make C compiler happy */
+    (void) cv;
+    
+    if (!perl_current_script)
+    {
+        perl_plugin->print_server (perl_plugin,
+                                   "Perl error: unable to add event handler, "
+                                   "script not initialized");
+	XSRETURN_NO;
+    }
+    
+    if (items < 2)
+    {
+        perl_plugin->print_server (perl_plugin,
+                                   "Perl error: wrong parameters for "
+                                   "\"add_event_handler\" function");
+        XSRETURN_NO;
+    }
+
+    event = SvPV (ST (0), PL_na);
+    function = SvPV (ST (1), PL_na);
+    
+    if (perl_plugin->event_handler_add (perl_plugin, event,
+                                        weechat_perl_event_handler, function,
+                                        (void *)perl_current_script))
+        XSRETURN_YES;
+    
+    XSRETURN_NO;
+}
+
 /*
  * weechat::remove_handler: remove a message/command handler
  */
@@ -844,7 +914,7 @@ static XS (XS_weechat_remove_handler)
     char *command, *function;
     dXSARGS;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) cv;
     
     if (!perl_current_script)
@@ -881,7 +951,7 @@ static XS (XS_weechat_remove_timer_handler)
     char *function;
     dXSARGS;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) cv;
     
     if (!perl_current_script)
@@ -917,7 +987,7 @@ static XS (XS_weechat_remove_keyboard_handler)
     char *function;
     dXSARGS;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) cv;
     
     if (!perl_current_script)
@@ -944,6 +1014,42 @@ static XS (XS_weechat_remove_keyboard_handler)
     XSRETURN_YES;
 }
 
+/*
+ * weechat::remove_event_handler: remove an event handler
+ */
+
+static XS (XS_weechat_remove_event_handler)
+{
+    char *function;
+    dXSARGS;
+    
+    /* make C compiler happy */
+    (void) cv;
+    
+    if (!perl_current_script)
+    {
+        perl_plugin->print_server (perl_plugin,
+                                   "Perl error: unable to remove event handler, "
+                                   "script not initialized");
+	XSRETURN_NO;
+    }
+    
+    if (items < 1)
+    {
+        perl_plugin->print_server (perl_plugin,
+                                   "Perl error: wrong parameters for "
+                                   "\"remove_event_handler\" function");
+        XSRETURN_NO;
+    }
+    
+    function = SvPV (ST (0), PL_na);
+    
+    weechat_script_remove_event_handler (perl_plugin, perl_current_script,
+                                         function);
+    
+    XSRETURN_YES;
+}
+
 /*
  * weechat::add_modifier: add a modifier
  */
@@ -953,7 +1059,7 @@ static XS (XS_weechat_add_modifier)
     char *type, *command, *function;
     dXSARGS;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) cv;
     
     if (!perl_current_script)
@@ -993,7 +1099,7 @@ static XS (XS_weechat_remove_modifier)
     char *type, *command, *function;
     dXSARGS;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) cv;
     
     if (!perl_current_script)
@@ -1031,7 +1137,7 @@ static XS (XS_weechat_get_info)
     char *arg, *info, *server_name;
     dXSARGS;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) cv;
     
     if (!perl_current_script)
@@ -1083,7 +1189,7 @@ static XS (XS_weechat_get_dcc_info)
     HV *dcc_hash_member;
     dXSARGS;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) cv;
     (void) items;
     
@@ -1144,7 +1250,7 @@ static XS (XS_weechat_get_config)
     char *option, *return_value;
     dXSARGS;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) cv;
     
     if (!perl_current_script)
@@ -1190,7 +1296,7 @@ static XS (XS_weechat_set_config)
     char *option, *value;
     dXSARGS;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) cv;
     
     if (!perl_current_script)
@@ -1230,7 +1336,7 @@ static XS (XS_weechat_get_plugin_config)
     char *option, *return_value;
     dXSARGS;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) cv;
     
     if (!perl_current_script)
@@ -1278,7 +1384,7 @@ static XS (XS_weechat_set_plugin_config)
     char *option, *value;
     dXSARGS;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) cv;
     
     if (!perl_current_script)
@@ -1322,7 +1428,7 @@ static XS (XS_weechat_get_server_info)
     HV *server_hash, *server_hash_member;
     dXSARGS;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) cv;
     (void) items;
     
@@ -1353,7 +1459,7 @@ static XS (XS_weechat_get_server_info)
 		 localtime(&ptr_server->away_time));
 	
 	server_hash_member = (HV *) sv_2mortal((SV *) newHV());	        
-        
+
 	hv_store (server_hash_member, "autoconnect",          11, newSViv (ptr_server->autoconnect), 0);
         hv_store (server_hash_member, "autoreconnect",        13, newSViv (ptr_server->autoreconnect), 0);
         hv_store (server_hash_member, "autoreconnect_delay",  19, newSViv (ptr_server->autoreconnect_delay), 0);
@@ -1379,12 +1485,14 @@ static XS (XS_weechat_get_server_info)
         hv_store (server_hash_member, "nick_modes",           10, newSVpv (ptr_server->nick_modes, 0), 0);
 	hv_store (server_hash_member, "away_time",             9, newSVpv (timebuffer, 0), 0);
 	hv_store (server_hash_member, "lag",                   3, newSViv (ptr_server->lag), 0);
-	
+
 	hv_store (server_hash, ptr_server->name, strlen(ptr_server->name), newRV_inc((SV *) server_hash_member), 0);	
-    }    
+    }
     perl_plugin->free_server_info (perl_plugin, server_info);
     
     ST (0) = newRV_inc((SV *) server_hash);
+    if (SvREFCNT(ST(0))) sv_2mortal(ST(0));
+    
     XSRETURN (1);
 }
 
@@ -1399,7 +1507,7 @@ static XS (XS_weechat_get_channel_info)
     HV *channel_hash, *channel_hash_member;
     dXSARGS;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) cv;
     
     if (!perl_current_script)
@@ -1451,6 +1559,8 @@ static XS (XS_weechat_get_channel_info)
     perl_plugin->free_channel_info (perl_plugin, channel_info);
     
     ST (0) = newRV_inc((SV *) channel_hash);
+    if (SvREFCNT(ST(0))) sv_2mortal(ST(0));
+
     XSRETURN (1);
 }
 
@@ -1465,7 +1575,7 @@ static XS (XS_weechat_get_nick_info)
     HV *nick_hash;
     dXSARGS;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) cv;
     
     if (!perl_current_script)
@@ -1515,6 +1625,8 @@ static XS (XS_weechat_get_nick_info)
     perl_plugin->free_nick_info (perl_plugin, nick_info);
     
     ST (0) = newRV_inc((SV *) nick_hash);
+    if (SvREFCNT(ST(0))) sv_2mortal(ST(0));
+    
     XSRETURN (1);
 }
 
@@ -1527,7 +1639,7 @@ static XS (XS_weechat_input_color)
     int color, start, length;
     dXSARGS;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) cv;
     
     if (!perl_current_script)
@@ -1565,7 +1677,7 @@ static XS (XS_weechat_get_irc_color)
     char *color;
     dXSARGS;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) cv;
     
     if (!perl_current_script)
@@ -1608,7 +1720,7 @@ static XS (XS_weechat_get_window_info)
     HV *window_hash_member;
     dXSARGS;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) cv;
     (void) items;
     
@@ -1656,7 +1768,7 @@ static XS (XS_weechat_get_buffer_info)
     char conv[8];
     dXSARGS;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) cv;
     (void) items;
     
@@ -1684,7 +1796,7 @@ static XS (XS_weechat_get_buffer_info)
     for (ptr_buffer = buffer_info; ptr_buffer; ptr_buffer = ptr_buffer->next_buffer)
     {
 	buffer_hash_member = (HV *) sv_2mortal((SV *) newHV());	        
-        
+
 	hv_store (buffer_hash_member, "type",           4, newSViv (ptr_buffer->type), 0);
         hv_store (buffer_hash_member, "num_displayed", 13, newSViv (ptr_buffer->num_displayed), 0);
 	hv_store (buffer_hash_member, "server",         6, 
@@ -1694,13 +1806,14 @@ static XS (XS_weechat_get_buffer_info)
         hv_store (buffer_hash_member, "notify_level",  12, newSViv (ptr_buffer->notify_level), 0);
 	hv_store (buffer_hash_member, "log_filename",  12, 
 		  newSVpv (ptr_buffer->log_filename == NULL ? "" : ptr_buffer->log_filename, 0), 0);
-	
 	snprintf(conv, sizeof(conv), "%d", ptr_buffer->number);
-	hv_store (buffer_hash, conv, strlen(conv), newRV_inc((SV *) buffer_hash_member), 0);	
+	hv_store (buffer_hash, conv, strlen(conv), newRV_inc((SV *) buffer_hash_member), 0);
     }    
     perl_plugin->free_buffer_info (perl_plugin, buffer_info);
     
     ST (0) = newRV_inc((SV *) buffer_hash);
+    if (SvREFCNT(ST(0))) sv_2mortal(ST(0));
+    
     XSRETURN (1);
 }
 
@@ -1718,7 +1831,7 @@ static XS (XS_weechat_get_buffer_data)
     
     dXSARGS;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) cv;
     (void) items;
     
@@ -1730,7 +1843,7 @@ static XS (XS_weechat_get_buffer_data)
 	XSRETURN_EMPTY;
     }
     
-    if (items > 2)
+    if (items != 2)
     {
         perl_plugin->print_server (perl_plugin,
                                    "Perl error: wrong parameters for "
@@ -1798,9 +1911,11 @@ weechat_perl_xs_init (pTHX)
     newXS ("weechat::add_command_handler", XS_weechat_add_command_handler, "weechat");
     newXS ("weechat::add_timer_handler", XS_weechat_add_timer_handler, "weechat");
     newXS ("weechat::add_keyboard_handler", XS_weechat_add_keyboard_handler, "weechat");
+    newXS ("weechat::add_event_handler", XS_weechat_add_event_handler, "weechat");
     newXS ("weechat::remove_handler", XS_weechat_remove_handler, "weechat");
     newXS ("weechat::remove_timer_handler", XS_weechat_remove_timer_handler, "weechat");
     newXS ("weechat::remove_keyboard_handler", XS_weechat_remove_keyboard_handler, "weechat");
+    newXS ("weechat::remove_event_handler", XS_weechat_remove_event_handler, "weechat");
     newXS ("weechat::add_modifier", XS_weechat_add_modifier, "weechat");
     newXS ("weechat::remove_modifier", XS_weechat_remove_modifier, "weechat");
     newXS ("weechat::get_info", XS_weechat_get_info, "weechat");
@@ -1825,6 +1940,7 @@ weechat_perl_xs_init (pTHX)
     newCONSTSUB (stash, "weechat::PLUGIN_RC_OK_IGNORE_WEECHAT", newSViv (PLUGIN_RC_OK_IGNORE_WEECHAT));
     newCONSTSUB (stash, "weechat::PLUGIN_RC_OK_IGNORE_PLUGINS", newSViv (PLUGIN_RC_OK_IGNORE_PLUGINS));
     newCONSTSUB (stash, "weechat::PLUGIN_RC_OK_IGNORE_ALL", newSViv (PLUGIN_RC_OK_IGNORE_ALL));
+    newCONSTSUB (stash, "weechat::PLUGIN_RC_OK_WITH_HIGHLIGHT", newSViv (PLUGIN_RC_OK_WITH_HIGHLIGHT));
 }
 
 /*
@@ -1874,7 +1990,7 @@ weechat_perl_load (t_weechat_plugin *plugin, char *filename)
         return 0;
     }
 
-    perl_current_script_filename = strdup (filename);
+    perl_current_script_filename = filename;
 
     PERL_SET_CONTEXT (perl_current_interpreter);
     perl_construct (perl_current_interpreter);
@@ -1885,9 +2001,6 @@ weechat_perl_load (t_weechat_plugin *plugin, char *filename)
     eval = weechat_perl_exec (plugin, &tempscript,
 			      SCRIPT_EXEC_INT,
 			      "weechat_perl_load_eval_file", filename, "", "");
-
-    free (perl_current_script_filename);
-
 #endif
     if (eval == NULL)
     {
@@ -2056,7 +2169,7 @@ weechat_perl_cmd (t_weechat_plugin *plugin,
     if (cmd_argc < 3)
         return PLUGIN_RC_KO;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) handler_args;
     (void) handler_pointer;
     
@@ -2164,9 +2277,28 @@ weechat_perl_cmd (t_weechat_plugin *plugin,
             if (!handler_found)
                 plugin->print_server (plugin, "  (none)");
             
+            /* list Perl event handlers */
+            plugin->print_server (plugin, "");
+            plugin->print_server (plugin, "Perl event handlers:");
+            handler_found = 0;
+            for (ptr_handler = plugin->handlers;
+                 ptr_handler; ptr_handler = ptr_handler->next_handler)
+            {
+                if ((ptr_handler->type == PLUGIN_HANDLER_EVENT)
+                    && (ptr_handler->handler_args))
+                {
+                    handler_found = 1;
+                    plugin->print_server (plugin, "  %s => Perl(%s)",
+                                          ptr_handler->event,
+                                          ptr_handler->handler_args);
+                }
+            }
+            if (!handler_found)
+                plugin->print_server (plugin, "  (none)");
+            
 	    /* List Perl modifiers */
 	    plugin->print_server (plugin, "");
-            plugin->print_server (plugin, "Python modifiers:");
+            plugin->print_server (plugin, "Perl modifiers:");
             modifier_found = 0;
             for (ptr_modifier = plugin->modifiers;
                  ptr_modifier; ptr_modifier = ptr_modifier->next_modifier)
