@@ -118,7 +118,7 @@ weechat_ruby_exec (t_weechat_plugin *plugin,
     VALUE rc, err;
     int ruby_error, *ret_i;
     void *ret_value;
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) plugin;
     
     ruby_current_script = script;
@@ -232,7 +232,7 @@ weechat_ruby_timer_handler (t_weechat_plugin *plugin,
                             int argc, char **argv,
                             char *handler_args, void *handler_pointer)
 {
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) argc;
     (void) argv;
     int *r;
@@ -281,6 +281,36 @@ weechat_ruby_keyboard_handler (t_weechat_plugin *plugin,
         return PLUGIN_RC_KO;
 }
 
+/*
+ * weechat_ruby_event_handler: general event handler for Ruby
+ */
+
+int
+weechat_ruby_event_handler (t_weechat_plugin *plugin,
+                            int argc, char **argv,
+                            char *handler_args, void *handler_pointer)
+{
+    int *r;
+    int ret;
+
+    if (argc >= 1)
+    {
+        r = (int *) weechat_ruby_exec (plugin, (t_plugin_script *)handler_pointer,
+                                       SCRIPT_EXEC_INT,
+                                       handler_args, argv[0], NULL, NULL);
+        if (r == NULL)
+            ret = PLUGIN_RC_KO;
+        else
+        {
+            ret = *r;
+            free (r);
+        }
+        return ret;
+    }
+    else
+        return PLUGIN_RC_KO;
+}
+
 /*
  * weechat_ruby_modifier: general modifier for Ruby
  */
@@ -308,7 +338,7 @@ weechat_ruby_register (int argc, VALUE *argv, VALUE class)
     VALUE name, version, shutdown_func, description, charset;
     char *c_name, *c_version, *c_shutdown_func, *c_description, *c_charset;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) class;
 
     ruby_current_script = NULL;
@@ -390,7 +420,7 @@ weechat_ruby_set_charset (VALUE class, VALUE charset)
 {
     char *c_charset;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) class;
     
     if (!ruby_current_script)
@@ -432,7 +462,7 @@ weechat_ruby_print (int argc, VALUE *argv, VALUE class)
     VALUE message, channel_name, server_name;
     char *c_message, *c_channel_name, *c_server_name;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) class;
     
     if (!ruby_current_script)
@@ -491,7 +521,7 @@ weechat_ruby_print_server (VALUE class, VALUE message)
 {
     char *c_message;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) class;
     
     if (!ruby_current_script)
@@ -530,7 +560,7 @@ weechat_ruby_print_infobar (VALUE class, VALUE delay, VALUE message)
     int c_delay;
     char *c_message;
 
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) class;
     
     if (!ruby_current_script)
@@ -573,7 +603,7 @@ weechat_ruby_remove_infobar (int argc, VALUE *argv, VALUE class)
     VALUE how_many;
     int c_how_many;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) class;
     
     if (!ruby_current_script)
@@ -611,7 +641,7 @@ weechat_ruby_log (int argc, VALUE *argv, VALUE class)
     VALUE message, channel_name, server_name;
     char *c_message, *c_channel_name, *c_server_name;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) class;
     
     if (!ruby_current_script)
@@ -671,7 +701,7 @@ weechat_ruby_command (int argc, VALUE *argv, VALUE class)
     VALUE command, channel_name, server_name;
     char *c_command, *c_channel_name, *c_server_name;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) class;
     
     if (!ruby_current_script)
@@ -730,7 +760,7 @@ weechat_ruby_add_message_handler (VALUE class, VALUE message, VALUE function)
 {
     char *c_message, *c_function;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) class;
     
     if (!ruby_current_script)
@@ -779,7 +809,7 @@ weechat_ruby_add_command_handler (int argc, VALUE *argv, VALUE class)
     char *c_command, *c_function, *c_description, *c_arguments, *c_arguments_description;
     char *c_completion_template;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) class;
     
     if (!ruby_current_script)
@@ -867,7 +897,7 @@ weechat_ruby_add_timer_handler (VALUE class, VALUE interval, VALUE function)
     int c_interval;
     char *c_function;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) class;
     
     if (!ruby_current_script)
@@ -913,7 +943,7 @@ weechat_ruby_add_keyboard_handler (VALUE class, VALUE function)
 {
     char *c_function;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) class;
     
     if (!ruby_current_script)
@@ -947,6 +977,52 @@ weechat_ruby_add_keyboard_handler (VALUE class, VALUE function)
     return INT2FIX (0);
 }
 
+/*
+ * weechat_ruby_add_event_handler: add a handler for events
+ */
+
+static VALUE
+weechat_ruby_add_event_handler (VALUE class, VALUE event, VALUE function)
+{
+    char *c_event, *c_function;
+    
+    /* make C compiler happy */
+    (void) class;
+    
+    if (!ruby_current_script)
+    {
+        ruby_plugin->print_server (ruby_plugin,
+                                   "Ruby error: unable to add event handler, "
+                                   "script not initialized");
+        return INT2FIX (0);
+    }
+    
+    c_event = NULL;
+    c_function = NULL;
+    
+    if (NIL_P (event) || NIL_P (function))
+    {
+        ruby_plugin->print_server (ruby_plugin,
+                                   "Ruby error: wrong parameters for "
+                                   "\"add_event_handler\" function");
+        return INT2FIX (0);
+    }
+    
+    Check_Type (event, T_STRING);
+    Check_Type (function, T_STRING);
+    
+    c_event = STR2CSTR (event);
+    c_function = STR2CSTR (function);
+    
+    if (ruby_plugin->event_handler_add (ruby_plugin, c_event,
+                                        weechat_ruby_event_handler,
+                                        c_function,
+                                        (void *)ruby_current_script))
+        return INT2FIX (1);
+    
+    return INT2FIX (0);
+}
+
 /*
  * weechat_ruby_remove_handler: remove a handler
  */
@@ -956,7 +1032,7 @@ weechat_ruby_remove_handler (VALUE class, VALUE command, VALUE function)
 {
     char *c_command, *c_function;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) class;
     
     if (!ruby_current_script)
@@ -999,7 +1075,7 @@ weechat_ruby_remove_timer_handler (VALUE class, VALUE function)
 {
     char *c_function;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) class;
     
     if (!ruby_current_script)
@@ -1039,7 +1115,7 @@ weechat_ruby_remove_keyboard_handler (VALUE class, VALUE function)
 {
     char *c_function;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) class;
     
     if (!ruby_current_script)
@@ -1070,6 +1146,46 @@ weechat_ruby_remove_keyboard_handler (VALUE class, VALUE function)
     return INT2FIX (1);
 }
 
+/*
+ * weechat_ruby_remove_event_handler: remove an event handler
+ */
+
+static VALUE
+weechat_ruby_remove_event_handler (VALUE class, VALUE function)
+{
+    char *c_function;
+    
+    /* make C compiler happy */
+    (void) class;
+    
+    if (!ruby_current_script)
+    {
+        ruby_plugin->print_server (ruby_plugin,
+                                   "Ruby error: unable to remove event handler, "
+                                   "script not initialized");
+        return INT2FIX (0);
+    }
+    
+    c_function = NULL;
+    
+    if (NIL_P (function))
+    {
+        ruby_plugin->print_server (ruby_plugin,
+                                   "Ruby error: wrong parameters for "
+                                   "\"remove_event_handler\" function");
+        return INT2FIX (0);
+    }
+    
+    Check_Type (function, T_STRING);
+    
+    c_function = STR2CSTR (function);
+    
+    weechat_script_remove_event_handler (ruby_plugin, ruby_current_script,
+                                         c_function);
+    
+    return INT2FIX (1);
+}
+
 /*
  * weechat_ruby_add_modifier: add a modifier
  */
@@ -1079,7 +1195,7 @@ weechat_ruby_add_modifier (VALUE class, VALUE type, VALUE message, VALUE functio
 {
     char *c_type, *c_message, *c_function;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) class;
     
     if (!ruby_current_script)
@@ -1128,7 +1244,7 @@ weechat_ruby_remove_modifier (VALUE class, VALUE type, VALUE command, VALUE func
 {
     char *c_type, *c_command, *c_function;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) class;
     
     if (!ruby_current_script)
@@ -1175,7 +1291,7 @@ weechat_ruby_get_info (int argc, VALUE *argv, VALUE class)
     char *c_arg, *c_server_name, *info;
     VALUE arg, server_name, return_value;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) class;
     
     if (!ruby_current_script)
@@ -1238,7 +1354,7 @@ weechat_ruby_get_dcc_info (VALUE class)
     char timebuffer2[64];
     struct in_addr in;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) class;
     
     if (!ruby_current_script)
@@ -1322,7 +1438,7 @@ weechat_ruby_get_config (VALUE class, VALUE option)
     char *c_option, *return_value;
     VALUE ruby_return_value;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) class;
     
     if (!ruby_current_script)
@@ -1370,7 +1486,7 @@ weechat_ruby_set_config (VALUE class, VALUE option, VALUE value)
 {
     char *c_option, *c_value;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) class;
     
     if (!ruby_current_script)
@@ -1417,7 +1533,7 @@ weechat_ruby_get_plugin_config (VALUE class, VALUE option)
     char *c_option, *return_value;
     VALUE ruby_return_value;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) class;
     
     if (!ruby_current_script)
@@ -1467,7 +1583,7 @@ weechat_ruby_set_plugin_config (VALUE class, VALUE option, VALUE value)
 {
     char *c_option, *c_value;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) class;
     
     if (!ruby_current_script)
@@ -1517,7 +1633,7 @@ weechat_ruby_get_server_info (VALUE class)
     VALUE server_hash, server_hash_member;
     char timebuffer[64];
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) class;
     
     if (!ruby_current_script)
@@ -1616,7 +1732,7 @@ weechat_ruby_get_channel_info (VALUE class, VALUE server)
     VALUE channel_hash, channel_hash_member;
     char *c_server;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) class;
         
     if (!ruby_current_script)
@@ -1689,7 +1805,7 @@ weechat_ruby_get_nick_info (VALUE class, VALUE server, VALUE channel)
     VALUE nick_hash, nick_hash_member;
     char *c_server, *c_channel;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) class;
 
     if (!ruby_current_script)
@@ -1757,7 +1873,7 @@ weechat_ruby_get_irc_color (VALUE class, VALUE color)
 {
     char *c_color;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) class;
     
     if (!ruby_current_script)
@@ -1795,7 +1911,7 @@ weechat_ruby_get_window_info (VALUE class)
     t_plugin_window_info *window_info, *ptr_window;
     VALUE window_list, window_list_member;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) class;
     
     if (!ruby_current_script)
@@ -1855,7 +1971,7 @@ weechat_ruby_get_buffer_info (VALUE class)
     t_plugin_buffer_info *buffer_info, *ptr_buffer;
     VALUE buffer_hash, buffer_hash_member;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) class;
     
     if (!ruby_current_script)
@@ -1907,15 +2023,14 @@ weechat_ruby_get_buffer_info (VALUE class)
  */
 
 static VALUE
-weechat_ruby_get_buffer_data (int argc, VALUE *argv, VALUE class)
+weechat_ruby_get_buffer_data (VALUE class, VALUE server, VALUE channel)
 {
     t_plugin_buffer_line *buffer_data, *ptr_data;
     VALUE data_list, data_list_member;
-    VALUE server, channel;
     char *c_server, *c_channel;
     char timebuffer[64];
 
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) class;
     
     if (!ruby_current_script)
@@ -1926,24 +2041,25 @@ weechat_ruby_get_buffer_data (int argc, VALUE *argv, VALUE class)
         return INT2FIX (0);
     }
 
-    server = Qnil;
-    channel = Qnil;
     c_server = NULL;
     c_channel = NULL;
 
-    rb_scan_args (argc, argv, "02", &server, &channel);
-    
-    if (!NIL_P (server))
+    if (NIL_P (server) || NIL_P (channel))
     {
-	Check_Type (server, T_STRING);
-	c_server = STR2CSTR (server);
+	ruby_plugin->print_server (ruby_plugin,
+                                   "Ruby error: wrong parameters for "
+                                   "\"get_buffer_data\" function");
+	return INT2FIX (0);
     }
     
-    if (!NIL_P (channel))
-    {
-	Check_Type (channel, T_STRING);
-	c_channel = STR2CSTR (channel);
-    }
+    Check_Type (server, T_STRING);
+    Check_Type (channel, T_STRING);
+
+    c_server = STR2CSTR (server);
+    c_channel = STR2CSTR (channel);
+
+    if (!c_server || !c_channel)
+	return INT2FIX (0);
     
     data_list = rb_ary_new();    
     if (NIL_P (data_list))
@@ -1986,7 +2102,8 @@ static VALUE
 weechat_ruby_output(VALUE self, VALUE str)
 {
     char *msg, *p, *m;
-    /* make gcc happy */
+    
+    /* make C compiler happy */
     (void) self;
     
     msg = strdup(STR2CSTR(str));
@@ -2027,7 +2144,7 @@ weechat_ruby_output(VALUE self, VALUE str)
 static VALUE 
 weechat_ruby_output_flush(VALUE self)
 {
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) self;
     
     return Qnil;
@@ -2062,13 +2179,11 @@ weechat_ruby_load (t_weechat_plugin *plugin, char *filename)
 
     curModule = rb_define_module(modname);
 
-    ruby_current_script_filename = strdup (filename);
+    ruby_current_script_filename = filename;
     
     ruby_retcode = rb_protect_funcall (curModule, rb_intern("load_eval_file"),
 				       &ruby_error, 1, rb_str_new2(filename));
     
-    free (ruby_current_script_filename);
-    
     if (ruby_retcode == Qnil) {
 	err = rb_inspect(rb_gv_get("$!"));
 	ruby_plugin->print_server (ruby_plugin, "Ruby error: \"%s\"", STR2CSTR(err));
@@ -2225,7 +2340,7 @@ weechat_ruby_cmd (t_weechat_plugin *plugin,
     t_plugin_handler *ptr_handler;
     t_plugin_modifier *ptr_modifier;
     
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) handler_args;
     (void) handler_pointer;
     
@@ -2336,6 +2451,25 @@ weechat_ruby_cmd (t_weechat_plugin *plugin,
             if (!handler_found)
                 plugin->print_server (plugin, "  (none)");
             
+            /* list Ruby event handlers */
+            plugin->print_server (plugin, "");
+            plugin->print_server (plugin, "Ruby event handlers:");
+            handler_found = 0;
+            for (ptr_handler = plugin->handlers;
+                 ptr_handler; ptr_handler = ptr_handler->next_handler)
+            {
+                if ((ptr_handler->type == PLUGIN_HANDLER_EVENT)
+                    && (ptr_handler->handler_args))
+                {
+                    handler_found = 1;
+                    plugin->print_server (plugin, "  %s => Ruby(%s)",
+                                          ptr_handler->event,
+                                          ptr_handler->handler_args);
+                }
+            }
+            if (!handler_found)
+                plugin->print_server (plugin, "  (none)");
+            
 	    /* list Ruby modifiers */
 	    plugin->print_server (plugin, "");
             plugin->print_server (plugin, "Ruby modifiers:");
@@ -2473,7 +2607,8 @@ weechat_plugin_init (t_weechat_plugin *plugin)
     rb_define_const(ruby_mWeechat, "PLUGIN_RC_KO", INT2NUM(PLUGIN_RC_KO));
     rb_define_const(ruby_mWeechat, "PLUGIN_RC_OK_IGNORE_WEECHAT", INT2NUM(PLUGIN_RC_OK_IGNORE_WEECHAT));
     rb_define_const(ruby_mWeechat, "PLUGIN_RC_OK_IGNORE_PLUGINS", INT2NUM(PLUGIN_RC_OK_IGNORE_PLUGINS));
-    rb_define_const(ruby_mWeechat, "PLUGIN_RC_OK_IGNORE_ALL", INT2NUM(PLUGIN_RC_OK_IGNORE_ALL));    
+    rb_define_const(ruby_mWeechat, "PLUGIN_RC_OK_IGNORE_ALL", INT2NUM(PLUGIN_RC_OK_IGNORE_ALL));
+    rb_define_const(ruby_mWeechat, "PLUGIN_RC_OK_WITH_HIGHLIGHT", INT2NUM(PLUGIN_RC_OK_WITH_HIGHLIGHT));
     rb_define_module_function (ruby_mWeechat, "register", weechat_ruby_register, -1);
     rb_define_module_function (ruby_mWeechat, "set_charset", weechat_ruby_set_charset, 1);
     rb_define_module_function (ruby_mWeechat, "print", weechat_ruby_print, -1);
@@ -2486,9 +2621,11 @@ weechat_plugin_init (t_weechat_plugin *plugin)
     rb_define_module_function (ruby_mWeechat, "add_command_handler", weechat_ruby_add_command_handler, -1);
     rb_define_module_function (ruby_mWeechat, "add_timer_handler", weechat_ruby_add_timer_handler, 2);
     rb_define_module_function (ruby_mWeechat, "add_keyboard_handler", weechat_ruby_add_keyboard_handler, 1);
+    rb_define_module_function (ruby_mWeechat, "add_event_handler", weechat_ruby_add_event_handler, 2);
     rb_define_module_function (ruby_mWeechat, "remove_handler", weechat_ruby_remove_handler, 2);
     rb_define_module_function (ruby_mWeechat, "remove_timer_handler", weechat_ruby_remove_timer_handler, 1);
     rb_define_module_function (ruby_mWeechat, "remove_keyboard_handler", weechat_ruby_remove_keyboard_handler, 1);
+    rb_define_module_function (ruby_mWeechat, "remove_event_handler", weechat_ruby_remove_event_handler, 1);
     rb_define_module_function (ruby_mWeechat, "add_modifier", weechat_ruby_add_modifier, 3);
     rb_define_module_function (ruby_mWeechat, "remove_modifier", weechat_ruby_remove_modifier, 3);
     rb_define_module_function (ruby_mWeechat, "get_info", weechat_ruby_get_info, -1);
@@ -2503,7 +2640,7 @@ weechat_plugin_init (t_weechat_plugin *plugin)
     rb_define_module_function (ruby_mWeechat, "get_irc_color", weechat_ruby_get_irc_color, 1);
     rb_define_module_function (ruby_mWeechat, "get_window_info", weechat_ruby_get_window_info, 0);
     rb_define_module_function (ruby_mWeechat, "get_buffer_info", weechat_ruby_get_buffer_info, 0);
-    rb_define_module_function (ruby_mWeechat, "get_buffer_data", weechat_ruby_get_buffer_data, -1);
+    rb_define_module_function (ruby_mWeechat, "get_buffer_data", weechat_ruby_get_buffer_data, 2);
 
     /* redirect stdin and stdout */
     ruby_mWeechatOutputs = rb_define_module("WeechatOutputs");
