@@ -326,7 +326,7 @@ weechat_script_print (t_weechat_plugin *plugin,
     
     buf2 = (script->charset && script->charset[0]) ?
         plugin->iconv_to_internal (plugin, script->charset, buf) : NULL;
-    plugin->print (plugin, server, channel, (buf2) ? buf2 : buf);
+    plugin->print (plugin, server, channel, "%s", (buf2) ? buf2 : buf);
     if (buf2)
         free (buf2);
 }
@@ -350,7 +350,7 @@ weechat_script_print_server (t_weechat_plugin *plugin,
     
     buf2 = (script->charset && script->charset[0]) ?
         plugin->iconv_to_internal (plugin, script->charset, buf) : NULL;
-    plugin->print_server (plugin, (buf2) ? buf2 : buf);
+    plugin->print_server (plugin, "%s", (buf2) ? buf2 : buf);
     if (buf2)
         free (buf2);
 }
@@ -374,7 +374,7 @@ weechat_script_print_infobar (t_weechat_plugin *plugin,
     
     buf2 = (script->charset && script->charset[0]) ?
         plugin->iconv_to_internal (plugin, script->charset, buf) : NULL;
-    plugin->print_infobar (plugin, time_displayed, (buf2) ? buf2 : buf);
+    plugin->print_infobar (plugin, time_displayed, "%s", (buf2) ? buf2 : buf);
     if (buf2)
         free (buf2);
 }
@@ -398,7 +398,7 @@ weechat_script_log (t_weechat_plugin *plugin,
     
     buf2 = (script->charset && script->charset[0]) ?
         plugin->iconv_to_internal (plugin, script->charset, buf) : NULL;
-    plugin->log (plugin, server, channel, (buf2) ? buf2 : buf);
+    plugin->log (plugin, server, channel, "%s", (buf2) ? buf2 : buf);
     if (buf2)
         free (buf2);
 }
@@ -516,6 +516,34 @@ weechat_script_remove_keyboard_handler (t_weechat_plugin *plugin,
     }
 }
 
+/*
+ * weechat_script_remove_event_handler: remove an event handler for a script
+ */
+
+void
+weechat_script_remove_event_handler (t_weechat_plugin *plugin,
+                                     t_plugin_script *script,
+                                     char *function)
+{
+    t_plugin_handler *ptr_handler, *next_handler;
+    
+    /* search and remove timer handlers */
+    ptr_handler = plugin->handlers;
+    while (ptr_handler)
+    {
+        if ((ptr_handler->type == PLUGIN_HANDLER_EVENT)
+            && ((t_plugin_script *)ptr_handler->handler_pointer == script)
+            && (plugin->ascii_strcasecmp (plugin, ptr_handler->handler_args, function) == 0))
+        {
+            next_handler = ptr_handler->next_handler;
+            plugin->handler_remove (plugin, ptr_handler);
+            ptr_handler = next_handler;
+        }
+        else
+            ptr_handler = ptr_handler->next_handler;
+    }
+}
+
 /*
  * weechat_script_remove_modifier: remove a modifier
  *                                 arg1=type, arg2=command, arg3=function
@@ -625,7 +653,7 @@ weechat_script_set_charset (t_weechat_plugin *plugin,
                             t_plugin_script *script,
                             char *charset)
 {
-    /* make gcc happy */
+    /* make C compiler happy */
     (void) plugin;
     
     if (script->charset)
